@@ -1,11 +1,11 @@
-# $Id: BaseObject.pm 513 2008-08-05 19:41:14Z miyagawa $
+# $Id: BaseObject.pm 573 2009-05-27 23:40:06Z athomason $
 
 package Data::ObjectDriver::BaseObject;
 use strict;
 use warnings;
 
 our $HasWeaken;
-eval q{ use Scalar::Util qw(weaken) };
+eval q{ use Scalar::Util qw(weaken) }; ## no critic
 $HasWeaken = !$@;
 
 use Carp ();
@@ -559,8 +559,12 @@ sub txn_active { $TransactionLevel }
 
 sub begin_work {
     my $class = shift;
-    if ($TransactionLevel > 0) {
-        warn __PACKAGE__ . ": one ore more transaction already active: $TransactionLevel";
+    if ( $TransactionLevel > 0 ) {
+        Carp::carp(
+            $TransactionLevel > 1
+            ? "$TransactionLevel transactions already active"
+            : "Transaction already active"
+        );
     }
     $TransactionLevel++;
 }
@@ -586,7 +590,7 @@ sub _end_txn {
     }
     
     if (! $TransactionLevel) {
-        warn __PACKAGE__ . ": no transaction active, ignored $meth";
+        Carp::carp("No active transaction to end; ignoring $meth");
         return;
     }
     my @wd = @WorkingDrivers;
@@ -612,8 +616,7 @@ sub inflate {
     my $class = shift;
     my($deflated) = @_;
     my $obj = $class->new;
-    $obj->set_values($deflated->{columns});
-    $obj->{changed_cols} = {};
+    $obj->set_values_internal($deflated->{columns});
     $obj->call_trigger('post_inflate');
     return $obj;
 }
@@ -760,7 +763,8 @@ Known C<column_defs> types are:
 =item * C<blob>
 
 A blob of binary data. C<Data::ObjectDriver::Driver::DBD::Pg> maps this to
-C<DBI::Pg::PG_BYTEA>, and C<DBD::SQLite> to C<DBI::SQL_BLOB>.
+C<DBI::Pg::PG_BYTEA>, C<DBD::SQLite> to C<DBI::SQL_BLOB> and C<DBD::Oracle>
+to C<ORA_BLOB>.
 
 =item * C<bin_char>
 
