@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Search.pm 3179 2008-11-11 04:18:13Z fumiakiy $
+# $Id: Search.pm 3696 2009-05-12 19:07:31Z jmarcotte $
 
 package MT::App::Search;
 
@@ -135,10 +135,13 @@ sub init_request {
 
     my $processed = 0;
     my $list      = {};
+	if ( my $blog_id = $q->param('blog_id') ) {
+	    $q->param('IncludeBlogs', $blog_id);
+    }
     if ( $app->run_callbacks( 'search_blog_list', $app, $list, \$processed ) )
     {
         if ($processed) {
-            $app->{searchparam}{IncludeBlogs} = $list;
+            $app->{searchparam}{IncludeBlogs} = $list if ($list && %$list);
         }
         else {
             my $blog_list = $app->create_blog_list(%no_override);
@@ -147,13 +150,8 @@ sub init_request {
                     && %$blog_list
                     && $blog_list->{IncludeBlogs}
                     && %{ $blog_list->{IncludeBlogs} };
-            if ( !exists( $app->{searchparam}{IncludeBlogs} )
-                && ( my $blog_id = $q->param('blog_id') ) )
-            {
-                $blog_id =~ s/\D//g;
-                $app->{searchparam}{IncludeBlogs}{$blog_id} = 1
-                    if $blog_id;
-            }
+            return $app->error( $app->translate('Invalid request.') )
+                if ! $processed && ( !exists $app->{searchparam}{IncludeBlogs} || !%{$app->{searchparam}{IncludeBlogs}} );
         }
     }
     else {
@@ -170,7 +168,7 @@ sub generate_cache_keys {
     $key .= lc($_) . encode_url( $q->param($_) ) foreach @p;
     $count_key .= lc($_) . encode_url( $q->param($_) )
         foreach grep { ( 'limit' ne lc($_) ) && ( 'offset' ne lc($_) ) } @p;
-    $app->{cache_keys} = { result => $key, count => $count_key };
+    $app->{cache_keys} = { result => $key, count => $count_key, content_type => "HTTP_CONTENT_TYPE::$key" };
 }
 
 sub init_cache_driver {
@@ -249,7 +247,7 @@ sub create_blog_list {
     ## If IncludeBlogs has not been set, we need to build a list of
     ## the blogs to search. If ExcludeBlogs was set, exclude any blogs
     ## set in that list from our final list.
-    unless ( exists $blog_list{IncludeBlogs} ) {
+    unless ( %{$blog_list{IncludeBlogs}} ) {
         my $exclude = $blog_list{ExcludeBlogs};
         my $iter    = $app->model('blog')->load_iter;
         while ( my $blog = $iter->() ) {
@@ -271,6 +269,10 @@ sub check_cache {
         if exists $cache->{ $app->{cache_keys}{count} };
     my $result = $cache->{ $app->{cache_keys}{result} }
         if exists $cache->{ $app->{cache_keys}{result} };
+    if ( exists $cache->{ $app->{cache_keys}{content_type} } ) {
+        my $content_type = $cache->{ $app->{cache_keys}{content_type} };
+        $app->{response_content_type} = $content_type;
+    }
 
     ( $count, $result );
 }
@@ -287,17 +289,18 @@ sub process {
         $app->run_callbacks( 'search_cache_hit', $count, $out );
         return $out;
     }
-
-    my @arguments = $app->search_terms();
-    return $app->error( $app->errstr ) if $app->errstr;
-
-    $count = 0;
     my $iter;
-    if (@arguments) {
-        ( $count, $iter ) = $app->execute(@arguments);
-        return $app->error( $app->errstr ) unless $iter;
+    if ( $app->param('searchTerms') || $app->param('search') ) {
+        my @arguments = $app->search_terms();
+        return $app->error( $app->errstr ) if $app->errstr;
 
-        $app->run_callbacks( 'search_post_execute', $app, \$count, \$iter );
+        $count = 0;
+        if (@arguments) {
+            ( $count, $iter ) = $app->execute(@arguments);
+            return $app->error( $app->errstr ) unless $iter;
+
+            $app->run_callbacks( 'search_post_execute', $app, \$count, \$iter );
+        }
     }
 
     my $format = q();
@@ -306,12 +309,19 @@ sub process {
             if $format !~ /\w+/;
     }
     my $method = "render";
-    $method .= $format if $format && $app->can( $method . $format );
+    if ( $format ) {
+        $method .= $format if $app->can( $method . $format );
+    }
+    elsif ( my $tmpl_name = $app->param('Template') ) {
+        $method .= $tmpl_name if $app->can( $method . $tmpl_name );
+    }
+
     $out = $app->$method( $count, $iter );
+    return $app->error( $app->errstr ) unless defined $out;
 
     my $result;
     if ( ref($out) && ( $out->isa('MT::Template') ) ) {
-        defined( $result = $app->build_page($out) )
+        defined( $result = $out->build() )
             or return $app->error( $out->errstr );
     }
     else {
@@ -359,8 +369,7 @@ sub search_terms {
     my $app = shift;
     my $q   = $app->param;
 
-    my $search_string = $q->param('searchTerms') || $q->param('search')
-        or return $app->errtrans('No search term was specified.');
+    my $search_string = $q->param('searchTerms') || $q->param('search');
     $app->{search_string} = $search_string;
     my $offset = $q->param('startIndex') || $q->param('offset') || 0;
     return $app->errtrans( 'Invalid value: [_1]', encode_html($offset) )
@@ -386,11 +395,14 @@ sub search_terms {
 
     my @terms;
     if (%def_terms) {
-
-       # If we have a term for the model's class column, add it separately, so
-       # array search() doesn't add the default class column term.
         my $type        = $app->{searchparam}{Type};
         my $model_class = MT->model($type);
+
+        delete $def_terms{blog_id}
+            unless $model_class->can('blog_id');
+
+        # If we have a term for the model's class column, add it separately, so
+        # array search() doesn't add the default class column term.
         if ( my $class_col = $model_class->properties->{class_column} ) {
             if ( $def_terms{$class_col} ) {
                 push @terms, { $class_col => delete $def_terms{$class_col} };
@@ -454,7 +466,7 @@ sub _cache_out {
 
     my $result;
     if ( ref($out) && ( $out->isa('MT::Template') ) ) {
-        defined( $result = $app->build_page($out) )
+        defined( $result = $out->build() )
             or die $out->errstr;
     }
     else {
@@ -464,14 +476,20 @@ sub _cache_out {
     my $cache_driver = $app->{cache_driver};
     $cache_driver->set( $app->{cache_keys}{result},
         $out, $app->config->SearchCacheTTL );
+    if ( exists( $app->{response_content_type} )
+      && ( 'text/html' ne $app->{response_content_type} ) )
+    {
+        $cache_driver->set( $app->{cache_keys}{content_type},
+            $app->{response_content_type}, $app->config->SearchCacheTTL );
+    }
 }
 
 sub _log_search {
     my ( $cb, $app, $count_ref, $iter_ref ) = @_;
 
     #FIXME: template name may not be 'feed' for search feed
-    unless ( $app->param('template')
-        && ( 'feed' eq $app->param('template') ) )
+    unless ( $app->param('Template')
+        && ( 'feed' eq $app->param('Template') ) )
     {
         my $blog_id = $app->first_blog_id();
         require MT::Log;
@@ -590,6 +608,7 @@ sub load_search_tmpl {
             or
             return $app->errtrans( "Opening local file '[_1]' failed: [_2]",
             $filename, "$!" );
+        $tmpl->text( $app->translate_templatized( $tmpl->text ) );
     }
     else {
 
@@ -607,6 +626,7 @@ sub load_search_tmpl {
             # load template from search_template path
             # template_paths method does the magic
             $tmpl = $app->load_tmpl( $app->config->SearchDefaultTemplate );
+            $tmpl->text( $app->translate_templatized( $tmpl->text ) );
         }
     }
     return $app->error( $app->errstr )
@@ -627,7 +647,7 @@ sub render {
         or return $app->error( $app->errstr );
     my $tmpl = $app->load_search_tmpl(@arguments)
         or return $app->error( $app->errstr );
-    $tmpl;
+    return $tmpl;
 }
 
 sub renderjs {
@@ -655,6 +675,18 @@ sub renderjs {
         { content => $content, next_url => $next_link } );
 }
 
+#FIXME: template name may not be 'feed' for search feed
+sub renderfeed {
+    my $app = shift;
+    my $tmpl = $app->render(@_);
+    my $out = $app->build_page($tmpl);
+    my $ctx = $tmpl->context;
+    if ( my $content_type = $ctx->stash('content_type') ) {
+        $app->{response_content_type} = $content_type;
+    }
+    return $out;
+}
+
 sub query_parse {
     my $app = shift;
     my (%columns) = @_;
@@ -963,6 +995,7 @@ sub _default_takedown {
     if ( my $cache_driver = $app->{cache_driver} ) {
         $cache_driver->purge_stale( 2 * $app->config->SearchCacheTTL );
     }
+    delete $app->{searchparam};
     1;
 }
 
@@ -1029,6 +1062,8 @@ whose priority is higher than itself.
 
     return $$result if defined $$result;
 
+=back
+
 =head1 AUTHOR & COPYRIGHT
 
 Please see L<MT/AUTHOR & COPYRIGHT>.
