@@ -1,4 +1,4 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -9,6 +9,7 @@ package MT::App::Wizard;
 use strict;
 use base qw( MT::App );
 
+use MT::I18N qw( encode_text );
 use MT::Util qw( trim );
 
 sub id {'wizard'}
@@ -24,6 +25,7 @@ sub init {
     $app->SUPER::init(@_);
     $app->{mt_dir} ||= $ENV{MT_HOME} || $param{Directory};
     $app->{is_admin} = 1;
+    $app->{plugin_template_path} = '';
     $app->add_methods(
         pre_start => \&pre_start,
         run_step  => \&run_step,
@@ -82,6 +84,15 @@ sub init_request {
         }
     }
 
+    # If mt-check.cgi exists, redirect to errro screen
+    my $cfg_exists = $app->is_config_exists();
+    if ($cfg_exists && lc $step ne 'seed' && lc $mode ne 'retry') {
+        my %param;
+        $param{cfg_exists} = 1;
+        $app->mode('pre_start');
+        return $app->build_page( "start.tmpl", \%param );
+    }
+
     $app->param( 'next_step', $new_step );
     $app->mode('run_step');
 }
@@ -134,6 +145,11 @@ sub init_core_registry {
                 label =>
                     'This module is needed if you wish to use the TrackBack system, the weblogs.com ping, or the MT Recently Updated ping.',
             },
+            'HTML::Parser' => {
+                link => 'http://search.cpan.org/dist/HTML-Parser',
+                label =>
+                    'HTML::Parser is optional; It is needed if you wish to use the TrackBack system, the weblogs.com ping, or the MT Recently Updated ping.',
+            },
             'SOAP::Lite' => {
                 link    => 'http://search.cpan.org/dist/SOAP-Lite',
                 version => 0.50,
@@ -165,6 +181,11 @@ sub init_core_registry {
                 label =>
                     'This module is needed if you would like to be able to create thumbnails of uploaded images.',
             },
+            'IPC::Run' => {
+                link => 'http://search.cpan.org/dist/IPC-Run',
+                label =>
+                    'This module is needed if you would like to be able to use NetPBM as the image driver for MT.',
+            },
             'Storable' => {
                 link => 'http://search.cpan.org/dist/Storable',
                 label =>
@@ -175,6 +196,11 @@ sub init_core_registry {
                 label =>
                     'This module accelerates comment registration sign-ins.',
             },
+            'Crypt::SSLeay' => {
+                link => 'http://search.cpan.org/dist/Crypt-SSLeay',
+                label =>
+                    'This module and its dependencies are required in order to allow commenters to be authenticated by OpenID providers such as AOL and Yahoo! which require SSL support.',
+            },
             'MIME::Base64' => {
                 link => 'http://search.cpan.org/dist/MIME-Base64',
                 label =>
@@ -318,15 +344,12 @@ sub pre_start {
     my %param;
 
     eval { use File::Spec; };
-    my ( $cfg, $cfg_exists, $static_file_path );
+    my ( $static_file_path );
     if ( !$@ ) {
-        $cfg = File::Spec->catfile( $app->{mt_dir}, 'mt-config.cgi' );
-        $cfg_exists |= 1 if -f $cfg;
-
         $static_file_path = File::Spec->catfile( $app->static_file_path );
     }
 
-    $param{cfg_exists}        = $cfg_exists;
+    $param{cfg_exists}        = $app->is_config_exists;
     $param{valid_static_path} = 1
         if $app->is_valid_static_path( $app->static_path );
     $param{mt_static_exists} = $app->mt_static_exists;
@@ -604,6 +627,7 @@ sub configure {
             $cfg->DBSocket( $param{dbsocket} ) if $param{dbsocket};
             $cfg->DBHost( $param{dbserver} )
                 if $param{dbserver} && ( $param{dbtype} ne 'oracle' );
+            my $current_charset = $cfg->PublishCharset;
             $cfg->PublishCharset( $param{publish_charset} )
                 if $param{publish_charset};
 
@@ -624,12 +648,20 @@ sub configure {
             # test loading of object driver with these parameters...
             require MT::ObjectDriverFactory;
             my $od = MT::ObjectDriverFactory->new($driver);
+
+            $cfg->PublishCharset( $current_charset );
+
             eval { $od->rw_handle; };    ## to test connection
             if ( my $err = $@ ) {
                 $err_msg
                     = $app->translate(
                     'An error occurred while attempting to connect to the database.  Check the settings and try again.'
                     );
+                if ( $param{publish_charset} ne $current_charset ) {
+                    # $param{publish_charset} is sometimes undef which forces encode_text
+                    # to guess_encode which should handle all of the cases.
+                    $err = encode_text( $err, $param{publish_charset}, $current_charset );
+                }
                 $err_more = $err;
             }
             else {
@@ -1069,6 +1101,19 @@ sub is_valid_static_path {
         and ( $response->content_length() != 0 )
         && ( $response->content =~ m/function\s+openManual/s );
 }
+
+sub is_config_exists {
+    my $app = shift;
+
+    eval { use File::Spec; };
+    my ( $cfg, $cfg_exists, $static_file_path );
+    if ( !$@ ) {
+        $cfg = File::Spec->catfile( $app->{mt_dir}, 'mt-config.cgi' );
+        $cfg_exists |= 1 if -f $cfg;
+    }
+    return $cfg_exists;
+}
+
 1;
 __END__
 
