@@ -1,4 +1,4 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -18,7 +18,10 @@ __PACKAGE__->install_properties( {
 } );
 
 # List of supported file extensions (to aid the stock 'can_handle' method.)
-sub extensions { [ qr/gif/i, qr/jpe?g/i, qr/png/i, ] }
+sub extensions {
+    my $pkg = shift;
+    return $pkg->SUPER::extensions( [ qr/gif/i, qr/jpe?g/i, qr/png/i, ] );
+}
 
 sub class_label {
     MT->translate('Image');
@@ -50,6 +53,9 @@ sub image_height {
 
     eval { require Image::Size; };
     return undef if $@;
+    if ( ! -e $asset->file_path || ! -r $asset->file_path ) {
+        return undef;
+    }
     my ( $w, $h, $id ) = Image::Size::imgsize($asset->file_path);
     $asset->meta('image_height', $h);
     if ($asset->id) {
@@ -65,6 +71,9 @@ sub image_width {
 
     eval { require Image::Size; };
     return undef if $@;
+    if ( ! -e $asset->file_path || ! -r $asset->file_path ) {
+        return undef;
+    }
     my ( $w, $h, $id ) = Image::Size::imgsize($asset->file_path);
     $asset->meta('image_width', $w);
     if ($asset->id) {
