@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Author.pm 2694 2008-07-03 03:44:32Z fumiakiy $
+# $Id: Author.pm 3691 2009-05-12 18:06:46Z jmarcotte $
 
 package MT::Author;
 
@@ -18,7 +18,6 @@ __PACKAGE__->install_properties({
         'type' => 'smallint not null',
         'email' => 'string(75)',
         'url' => 'string(255)',
-        'hint' => 'string(75)',
         'public_key' => 'text',
         'preferred_language' => 'string(50)',
         'api_password' => 'string(60)',
@@ -38,9 +37,15 @@ __PACKAGE__->install_properties({
         'userpic_asset_id' => 'integer',
         'basename' => 'string(255)',
 
+        # Deprecated; The hint is not used from 4.25 in the password recovery
+        'hint' => 'string(75)',
+
         # meta properties
         'widgets' => 'hash meta',
         'favorite_blogs' => 'array meta',
+        'password_reset' => 'string meta',
+        'password_reset_expires' => 'string meta',
+        'password_reset_return_to' => 'string meta',
     },
     defaults => {
         type => 1,
@@ -103,15 +108,10 @@ sub set_defaults {
 
 sub remove_sessions {
     my $auth = shift;
+    return if ( ! $auth or ! $auth->id );
     require MT::Session;
-    my $sess_iter = MT::Session->load_iter({ kind => 'US' });
-    my @sess;
-    while (my $sess = $sess_iter->()) {
-        my $id = $sess->get('author_id');
-        next unless $id == $auth->id;
-        push @sess, $sess;
-    }
-    $_->remove foreach @sess;
+    my $sess_iter = MT::Session->remove({ kind => 'US', name => $auth->id });
+    return 1;
 }
 
 sub set_password {
@@ -160,13 +160,13 @@ sub set_commenter_perm {
     }
     if ($action eq 'approve') {
         $perm->remove_restrictions('comment');
-        $perm->can_comment(1) if COMMENTER eq $this->type;
+        $perm->can_comment(1);
     } elsif (($action eq 'ban') || ($action eq 'block')) {
         $perm->set_these_restrictions('comment');
-        $perm->can_comment(0) if COMMENTER eq $this->type;
+        $perm->can_comment(0);
     } elsif ($action eq 'pending') {
         $perm->remove_restrictions('comment');
-        $perm->can_comment(0) if COMMENTER eq $this->type;
+        $perm->can_comment(0);
     }
     $perm->save()
         or return $this->error(MT->translate("The approval could not be committed: [_1]", $perm->errstr));
@@ -232,8 +232,10 @@ sub save {
             }
         }
         # Generate basename
-        my $basename = MT::Util::make_unique_author_basename($auth);
-        $auth->basename($basename);
+        unless ($auth->basename()) {
+            my $basename = MT::Util::make_unique_author_basename($auth);
+            $auth->basename($basename);
+        }
     }
 
     my $privs;
@@ -285,8 +287,7 @@ sub is_superuser {
             $author->permissions(0)->can_edit_templates(@_);
         }
     } else {
-        $author->permissions(0)->can_administer() ||
-            $author->SUPER::is_superuser();
+        $author->permissions(0)->can_administer();
     }
 }
 
@@ -431,14 +432,13 @@ sub can_administer {
 
 sub entry_prefs {
     my $author = shift;
-    my @prefs = split /,/, ($author->column('entry_prefs') || '');
+    my @prefs = split /,/, ((@_ ? $_[0] : $author->column('entry_prefs')) || '');
     my %prefs;
     foreach (@prefs) {
         my ($name, $value) = split /=/, $_, 2;
         $prefs{$name} = $value;
     }
     if (@_) {
-        %prefs = (%prefs, @_);
         my $pref = '';
         foreach (keys %prefs) {
             $pref .= ',' if $pref ne '';
@@ -735,7 +735,7 @@ sub userpic_url {
         $author->userpic_thumbnail_options(),
         %param,
     );
-    if ($info[0] !~ m!^https?://!) {
+    if (($info[0] || '') !~ m!^https?://!) {
         my $static_host = MT->instance->static_path;
         if ($static_host =~ m!^https?://!) {
             $static_host =~ s!^(https?://[^/]+?)!$1!;
@@ -854,6 +854,7 @@ The author's homepage URL.
 =item * hint
 
 The answer to the question used when recovering the user's password.
+The hint is not used from 4.25 in the password recovery.
 
 =item * external_id
 
