@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: BackupRestore.pm 2730 2008-07-08 07:58:50Z fumiakiy $
+# $Id: BackupRestore.pm 3703 2009-05-12 20:26:18Z jmarcotte $
 
 package MT::BackupRestore;
 use strict;
@@ -92,6 +92,13 @@ sub core_backup_instructions {
         'ts_funcmap'    => {
             'skip' => 1
         },
+        # group and touch object should be skipped too; touch object creates issues with restore process (https://roundup.apperceptive.com/issues/issue11526)
+        'touch' => {
+            'skip' => 1
+        },
+        'group' => {
+            'skip' => 1
+        },
     };
 }
 
@@ -218,8 +225,11 @@ sub _default_terms_args {
 sub backup {
     my $class = shift;
     my ($blog_ids, $printer, $splitter, $finisher, $progress, $size, $enc, $metadata) = @_;
-    push @$blog_ids, '0'
-        if defined($blog_ids) && scalar(@$blog_ids);
+
+	# removed global items from backup for blog-specific backups as this creates multiple copies of global items
+	# when multiple blogs from the same instance are restored one-by-one. ideally, this should be a user setting on the backup form
+	# push @$blog_ids, '0' if defined($blog_ids) && scalar(@$blog_ids);
+
     my $obj_to_backup = $class->_populate_obj_to_backup( $blog_ids );
 
     my $header .= "<movabletype xmlns='" . NS_MOVABLETYPE . "'\n";
@@ -787,8 +797,12 @@ sub to_xml {
     unless ( UNIVERSAL::isa( $obj, 'MT::Log' ) ) {
         if ( $obj->properties
           && ( my $ccol = $obj->properties->{class_column} ) ) {
-            my $class = $obj->$ccol;
-            $elem = $class if $class;
+            if ( my $class = $obj->$ccol ) {
+                # use class_type value instead if
+                # the value resolves to a Perl package
+                $elem = $class
+                    if defined( MT->model($class) );
+            }
         }
     }
 
@@ -1294,6 +1308,24 @@ sub parents {
     };
 }
 
+package MT::IPBanList;
+
+sub parents {
+    my $obj = shift;
+    {
+        blog_id => MT->model('blog'),
+    };
+}
+
+package MT::Blocklist;
+
+sub parents {
+    my $obj = shift;
+    {
+        blog_id => MT->model('blog'),
+    };
+}
+
 1;
 __END__
 
@@ -1450,6 +1482,8 @@ $asset has new url and path.
 $callback is a code reference which will print out the passed paramter.
 Callback method can use this to communicate with users.
 
+=back
+
 =head1 AUTHOR & COPYRIGHT
 
 Please see L<MT/AUTHOR & COPYRIGHT>.
