@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Blog.pm 3006 2008-09-01 02:24:56Z fumiakiy $
+# $Id: Blog.pm 3803 2009-06-08 03:20:26Z fumiakiy $
 
 package MT::Blog;
 
@@ -177,8 +177,8 @@ sub set_defaults {
         ping_blogs => 0,
         ping_technorati => 0,
         ping_google => 0,
-        archive_type => 'Individual,Monthly,Category,Category-Monthly,Page',
-        archive_type_preferred => 'Individual',
+        archive_type => '',
+        archive_type_preferred => '',
         status_default => 2,
         junk_score_threshold => 0,
         junk_folder_expiry => 14, # 14 days
@@ -221,10 +221,16 @@ sub create_default_blog {
 sub create_default_templates {
     my $blog = shift;
 
+    my $app = MT->instance;
+    my $curr_lang = $app->current_language;
+    $app->set_language($blog->language);
+
     require MT::DefaultTemplates;
     my $tmpl_list = MT::DefaultTemplates->templates( @_ );
-    return $blog->error(MT->translate("No default templates were found."))
-        if !$tmpl_list || (ref($tmpl_list) ne 'ARRAY') || (!@$tmpl_list);
+    if ( !$tmpl_list || (ref($tmpl_list) ne 'ARRAY') || (!@$tmpl_list) ) {
+        $app->set_language($curr_lang);
+        return $blog->error(MT->translate("No default templates were found."));
+    }
 
     require MT::Template;
     my @arch_tmpl;
@@ -258,6 +264,7 @@ sub create_default_templates {
         }
     }
 
+    my %archive_types;
     if (@arch_tmpl) {
         require MT::TemplateMap;
         for my $map_set (@arch_tmpl) {
@@ -266,6 +273,7 @@ sub create_default_templates {
             foreach my $map_key (keys %$mappings) {
                 my $m = $mappings->{$map_key};
                 my $at = $m->{archive_type};
+                $archive_types{$at} = 1;
                 # my $preferred = $mappings->{$map_key}{preferred};
                 my $map = MT::TemplateMap->new;
                 $map->archive_type($at);
@@ -283,6 +291,11 @@ sub create_default_templates {
         }
     }
 
+    $blog->archive_type( join ',', keys %archive_types );
+    foreach my $at ( qw( Individual Daily Weekly Monthly Category ) ) {
+        $blog->archive_type_preferred($at), last
+            if exists $archive_types{$at};
+    }
     $blog->custom_dynamic_templates('none');
     $blog->save;
 
@@ -292,6 +305,7 @@ sub create_default_templates {
         $tmpl_list
     );
 
+    $app->set_language($curr_lang);
     return $blog;
 }
 
@@ -485,13 +499,7 @@ sub has_archive_type {
     my $blog = shift;
     my ($type) = @_;
     my %at = map { lc $_ => 1 } split(/,/, $blog->archive_type);
-    my $has = exists $at{lc $type} ? 1 : 0;
-    if ($has) {
-        my $map_class = MT->model('templatemap');
-        my $map = $map_class->load({ archive_type => $type, blog_id => $blog->id });
-        $has = defined $map ? 1 : 0;
-    }
-    return $has;
+    return exists $at{lc $type} ? 1 : 0;
 }
 
 sub accepts_registered_comments {
@@ -568,7 +576,7 @@ sub clone_with_children {
 
     # Cloning blog
     my $new_blog = $blog->clone($params);
-    $new_blog->name(MT->translate($blog_name ? $blog_name : "Clone of [_1]", $blog->name));
+    $new_blog->name($blog_name ? $blog_name : MT->translate("Clone of [_1]", $blog->name));
     delete $new_blog->{column_values}->{id};
     delete $new_blog->{changed_cols}->{id};
     $new_blog->save or die $new_blog->errstr;
@@ -872,8 +880,18 @@ sub clone_with_children {
             my @old_widgets = split /,/, $tmpl->modulesets;
             $tmpl_processor->($new_blog_id, \$counter, $tmpl, \%tmpl_map);
             my @new_widgets;
-            push @new_widgets, $tmpl_map{$_}
-                foreach @old_widgets;
+            foreach ( @old_widgets ) {
+                if ( exists $tmpl_map{$_} ) {
+                    push @new_widgets, $tmpl_map{$_};
+                }
+                else {
+                    my $global_widget = MT::Template->load( $_ );
+                    push @new_widgets, $_
+                      if $global_widget
+                      && $global_widget->blog_id == 0
+                      && $global_widget->type eq 'widget'
+                }
+            }
             $tmpl->modulesets( join(',', @new_widgets) );
             $tmpl->save;
         }
