@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Bootstrap.pm 3179 2008-11-11 04:18:13Z fumiakiy $
+# $Id: Bootstrap.pm 3829 2009-06-10 03:45:26Z takayama $
 
 package MT::Bootstrap;
 
@@ -34,6 +34,23 @@ sub BEGIN {
         if $orig_dir && ($orig_dir ne $dir);
 }
 
+my $fcgi_exit_requested = 0;
+my $fcgi_handling_request = 0;
+
+sub fcgi_sig_handler {
+    my $sig = shift;
+    $fcgi_exit_requested = $sig;
+    if ($fcgi_handling_request) {
+        # With exit requested flag set, FastCGI loop will exit when it is done.
+        print STDERR "Movable Type: SIG$sig caught. Exiting gracefully after current request.\n";
+    }
+    else {
+        # Not currently handling a request, so just go ahead and exit.
+        print STDERR "Movable Type: SIG$sig caught. Exiting gracefully.\n";
+        exit(1);
+    }
+}
+
 sub import {
     my ($pkg, %param) = @_;
 
@@ -47,11 +64,11 @@ sub import {
         my $not_fast_cgi = 0;
         $not_fast_cgi ||= exists $ENV{$_}
             for qw(HTTP_HOST GATEWAY_INTERFACE SCRIPT_FILENAME SCRIPT_URL);
-         my $fast_cgi = defined $param{FastCGI} ? $param{FastCGI} : (!$not_fast_cgi);
-         if ($fast_cgi) {
-             eval 'require CGI::Fast;';
-             $fast_cgi = 0 if $@;
-         }
+        my $fast_cgi = defined $param{FastCGI} ? $param{FastCGI} : (!$not_fast_cgi);
+        if ($fast_cgi) {
+            eval 'require CGI::Fast;';
+            $fast_cgi = 0 if $@;
+        }
 
         # ready to run now... run inside an eval block so we can gracefully
         # die if something bad happens
@@ -62,13 +79,54 @@ sub import {
             eval "# line " . __LINE__ . " " . __FILE__ . "\nrequire $class; 1;" or die $@;
             if ($fast_cgi) {
                 $ENV{FAST_CGI} = 1;
-                while (my $cgi = new CGI::Fast) {
+                # Signal handling needs FAIL_ACCEPT_ON_INTR set:
+                # "If set, Accept will fail if interrupted. It not set, it
+                # will just keep on waiting."
+                require FCGI;
+                $CGI::Fast::Ext_Request = FCGI::Request( \*STDIN, \*STDOUT, \*STDERR, \%ENV, 0, FCGI::FAIL_ACCEPT_ON_INTR());
+                my ($max_requests, $max_time, $cfg);
+                # catch SIGUSR1 and SIGTERM and allow request to finish before
+                # exiting.
+                # TODO: handle SIGPIPE more gracefully.
+                $SIG{USR1} = \&fcgi_sig_handler;
+                $SIG{TERM} = \&fcgi_sig_handler;
+                $SIG{PIPE} = 'IGNORE';
+                # we set the "handling request" flag so the signal handler can exit
+                # immediately when requests aren't being handled.
+                while ($fcgi_handling_request = (my $cgi = new CGI::Fast)) {
                     $app = $class->new( %param, CGIObject => $cgi )
                         or die $class->errstr;
+
+                    $app->{fcgi_startup_time} ||= time;
+                    $app->{fcgi_request_count} = ( $app->{fcgi_request_count} || 0 ) + 1;
+
+                    unless ( $cfg ) {
+                        $cfg = $app->config;
+                        $max_requests = $cfg->FastCGIMaxRequests;
+                        $max_time = $cfg->FastCGIMaxTime;
+                    }
+
                     local $SIG{__WARN__} = sub { $app->trace($_[0]) };
                     MT->set_instance($app);
                     $app->init_request(CGIObject => $cgi);
                     $app->run;
+                    # force closing of connection here
+                    $CGI::Fast::Ext_Request->Finish();
+
+                    $fcgi_handling_request = 0;
+                    # Check for caught signal
+                    if ( $fcgi_exit_requested ) {
+                        print STDERR "Movable Type: FastCGI request loop exiting. Caught signal SIG$fcgi_exit_requested.\n";
+                        last;
+                    }
+                    # Check for timeout for this process
+                    elsif ( $max_time && ( time - $app->{fcgi_startup_time} >= $max_time ) ) {
+                        last;
+                    }
+                    # Check for max executions for this process
+                    elsif ( $max_requests && ( $app->{fcgi_request_count} >= $max_requests ) ) {
+                        last;
+                    }
                 }
             } else {
                 $app = $class->new( %param ) or die $class->errstr;
@@ -77,6 +135,31 @@ sub import {
             }
         };
         if (my $err = $@) {
+            if (!$app && $err =~ m/Missing configuration file/) {
+                my $host = $ENV{SERVER_NAME} || $ENV{HTTP_HOST};
+                $host =~ s/:\d+//;
+                my $port = $ENV{SERVER_PORT};
+                my $uri = $ENV{REQUEST_URI} || $ENV{SCRIPT_NAME};
+                if ($uri =~ m/(\/mt\.(f?cgi|f?pl)(\?.*)?)$/) {
+                    my $script = $1;
+                    my $ext = $2;
+
+                    if (-f File::Spec->catfile($ENV{MT_HOME}, "mt-wizard.$ext")) {
+                        $uri =~ s/\Q$script\E//;
+                        $uri .= '/mt-wizard.' . $ext;
+
+                        my $prot = $port == 443 ? 'https' : 'http';
+                        my $cgipath = "$prot://$host";
+                        $cgipath .= ":$port"
+                            unless $port == 443 or $port == 80;
+                        $cgipath .= $uri;
+                        print "Status: 302 Moved\n";
+                        print "Location: " . $cgipath . "\n\n";
+                        exit;
+                    }
+                }
+            }
+
             my $charset = 'utf-8';
             eval {
                 # line __LINE__ __FILE__
@@ -107,31 +190,6 @@ sub import {
                     exit;
                 };
                 $err = $@;
-            } else {
-                if ($err =~ m/Missing configuration file/) {
-                    my $host = $ENV{SERVER_NAME} || $ENV{HTTP_HOST};
-                    $host =~ s/:\d+//;
-                    my $port = $ENV{SERVER_PORT};
-                    my $uri = $ENV{REQUEST_URI} || $ENV{SCRIPT_NAME};
-                    if ($uri =~ m/(\/mt\.(f?cgi|f?pl)(\?.*)?)$/) {
-                        my $script = $1;
-                        my $ext = $2;
-
-                        if (-f File::Spec->catfile($ENV{MT_HOME}, "mt-wizard.$ext")) {
-                            $uri =~ s/\Q$script\E//;
-                            $uri .= '/mt-wizard.' . $ext;
-
-                            my $prot = $port == 443 ? 'https' : 'http';
-                            my $cgipath = "$prot://$host";
-                            $cgipath .= ":$port"
-                                unless $port == 443 or $port == 80;
-                            $cgipath .= $uri;
-                            print "Status: 302 Moved\n";
-                            print "Location: " . $cgipath . "\n\n";
-                            exit;
-                        }
-                    }
-                }
             }
             if (!$MT::DebugMode && ($err =~ m/^(.+?)( at .+? line \d+)(.*)$/s)) {
                 $err = $1;
