@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Builder.pm 2212 2008-05-02 22:03:06Z auno $
+# $Id: Builder.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::Builder;
 
@@ -69,12 +69,13 @@ sub compile {
     # Other namespaces (like 'Foo') would require the colon.
     # Tag and attributes are case-insensitive. So you can write:
     #   <mtfoo>...</MTFOO>
-    while ($text =~ m!(<\$?(MT:?)((?:<[^>]+?>|"(?:<[^>]+?>|.)*?"|'(?:<[^>]+?>|.)*?'|.)+?)[\$/]?>)!gis) {
-        my($whole_tag, $prefix, $tag) = ($1, $2, $3);
+    while ($text =~ m!(<\$?(MT:?)((?:<[^>]+?>|"(?:<[^>]+?>|.)*?"|'(?:<[^>]+?>|.)*?'|.)+?)([-]?)[\$/]?>)!gis) {
+        my($whole_tag, $prefix, $tag, $space_eater) = ($1, $2, $3, $4);
         ($tag, my($args)) = split /\s+/, $tag, 2;
         my $sec_start = pos $text;
         my $tag_start = $sec_start - length $whole_tag;
         _text_block($state, $pos, $tag_start) if $pos < $tag_start;
+        $state->{space_eater} = $space_eater;
         $args ||= '';
         # Structure of a node:
         #   tag name, attribute hashref, contained tokens, template text,
@@ -256,6 +257,8 @@ sub _consume_up_to {
 sub _text_block {
     my $text = substr ${ $_[0]->{text} }, $_[1], $_[2] - $_[1];
     if ((defined $text) && ($text ne '')) {
+        return if $_[0]->{space_eater} && ($text =~ m/^\s+$/s);
+        $text =~ s/^\s+//s if $_[0]->{space_eater};
         my $rec = bless [ 'TEXT', $text, undef, undef, undef, $_[0]->{tokens}, $_[0]->{tmpl} ], NODE;
         # Avoids circular reference between NODE and TOKENS, MT::Template.
         weaken($rec->[5]);
@@ -338,6 +341,8 @@ sub build {
                 $uncompiled = $t->[3];
             }
             my($h, $type) = $ctx->handler_for($t->[0]);
+            my $conditional = defined $type && $type == 2;
+
             if ($h) {
                 $timer->pause_partial if $timer;
                 local($ctx->{__stash}{tag}) = $t->[0];
@@ -352,7 +357,7 @@ sub build {
                     if (ref $args{$v} eq 'ARRAY') {
                         foreach (@{$args{$v}}) {
                             if (m/^\$([A-Za-z_](\w|\.)*)$/) {
-                                $_ = $ctx->var($1);
+                                local $_ = $ctx->var($1);
                             }
                         }
                     } else {
@@ -381,6 +386,13 @@ sub build {
                 # Stores a reference to the ordered list of arguments,
                 # just in case the handler wants them
                 local $args{'@'} = \@args;
+
+                my $vars = $ctx->{__stash}{vars};
+                local $vars->{__cond_value__} = $vars->{__cond_value__}
+                    if $conditional;
+                local $vars->{__cond_name__}  = $vars->{__cond_name__}
+                    if $conditional;
+
                 my $out = $h->($ctx, \%args, $cond);
 
                 unless (defined $out) {
@@ -395,11 +407,8 @@ sub build {
                     }
                 }
 
-                if ((defined $type) && ($type == 2)) {
+                if ($conditional) {
                     # conditional; process result
-                    my $vars = $ctx->{__stash}{vars};
-                    local $vars->{__value__} = delete $vars->{__cond_value__};
-                    local $vars->{__name__}  = delete $vars->{__cond_name__};
                     $out = $out ? $ctx->slurp(\%args, $cond) : $ctx->else(\%args, $cond);
                     delete $vars->{__cond_tag__};
                     return $build->error(MT->translate("Error in <mt[_1]> tag: [_2]", $t->[0], $ctx->errstr))
