@@ -46,10 +46,20 @@ sub edit {
         $param->{ "moderate_pings_" . ( $obj->moderate_pings || 0 ) } = 1;
 
         my $cmtauth_reg = $app->registry('commenter_authenticators');
+        my %cmtauth;
         foreach my $auth ( keys %$cmtauth_reg ) {
-            $cmtauth_reg->{$auth}->{disabled} = 1
-              if exists( $cmtauth_reg->{$auth}->{condition} )
-              && !( $cmtauth_reg->{$auth}->{condition}->() );
+            my %auth = %{$cmtauth_reg->{$auth}};
+            $cmtauth{$auth} = \%auth;
+            if ( my $c = $cmtauth_reg->{$auth}->{condition} ) {
+                $c = $app->handler_to_coderef($c);
+                if ( $c ) {
+                    my $reason;
+                    $cmtauth{$auth}->{disabled} = 1 unless $c->( $blog, \$reason );
+                    $cmtauth{$auth}->{disabled_reason} = $reason if $reason;
+                    delete $cmtauth{TypeKey}
+                        if $auth eq 'TypeKey' && $cmtauth{TypeKey}{disabled};
+                }
+            }
         }
         if ( my $auths = $blog->commenter_authenticators ) {
             foreach ( split ',', $auths ) {
@@ -57,31 +67,25 @@ sub edit {
                     $param->{enabled_MovableType} = 1;
                 }
                 else {
-                    $cmtauth_reg->{$_}->{enabled} = 1;
+                    $cmtauth{$_}->{enabled} = 1;
                 }
             }
         }
         my @cmtauth_loop;
-        foreach ( keys %$cmtauth_reg ) {
-            $cmtauth_reg->{$_}->{key} = $_;
+        foreach ( keys %cmtauth ) {
+            $cmtauth{$_}->{key} = $_;
             if (
                 UNIVERSAL::isa(
-                    $cmtauth_reg->{$_}->{plugin}, 'MT::Plugin'
+                    $cmtauth{$_}->{plugin}, 'MT::Plugin'
                 )
               )
             {
-                push @cmtauth_loop, $cmtauth_reg->{$_};
+                # force plugin auth schemes to show after native auth schemes
+                $cmtauth{$_}{order} = ($cmtauth{$_}{order} || 0) + 100;
             }
+            push @cmtauth_loop, $cmtauth{$_};
         }
-        unshift @cmtauth_loop, $cmtauth_reg->{'TypeKey'}
-          if exists( $cmtauth_reg->{'TypeKey'} )
-          && $blog->remote_auth_token;
-        unshift @cmtauth_loop, $cmtauth_reg->{'Vox'}
-          if exists $cmtauth_reg->{'Vox'};
-        unshift @cmtauth_loop, $cmtauth_reg->{'LiveJournal'}
-          if exists $cmtauth_reg->{'LiveJournal'};
-        unshift @cmtauth_loop, $cmtauth_reg->{'OpenID'}
-          if exists $cmtauth_reg->{'OpenID'};
+        @cmtauth_loop = sort { $a->{order} <=> $b->{order} } @cmtauth_loop;
 
         $param->{cmtauth_loop} = \@cmtauth_loop;
 
@@ -296,6 +300,8 @@ sub edit {
         @$sets = sort { $a->{order} <=> $b->{order} } @$sets;
         $param->{'template_set_loop'} = $sets;
         $param->{'template_set_index'} = $#$sets;
+
+        $param->{languages} = $app->languages_list( $app->current_language );
     }
 
     if (   !$param->{site_path}
@@ -1337,11 +1343,6 @@ sub pre_save {
               keys %$ping_servers;
             $obj->update_pings( join( ',', @pings_list ) );
         }
-        if ( $screen eq 'cfg_entry' ) {
-            my %param = $_[0] ? %{ $_[0] } : ();
-            my $pref_param = $app->load_entry_prefs;
-            %param = ( %param, %$pref_param );
-        }
         if ( $screen eq 'cfg_trackbacks' ) {
             if ( my $pings = $app->param('allow_pings') ) {
                 if ($pings) {
@@ -1553,7 +1554,7 @@ sub post_save {
         if ($path_changed) {
             update_dynamicity( $app, $obj );
             $app->rebuild( BlogID => $obj->id, NoStatic => 1 )
-                or return $app->publish_error();
+                or $app->publish_error();
         }
 
         cfg_archives_save($app, $obj) or return;
@@ -1816,65 +1817,48 @@ sub build_blog_table {
             description => $blog->description,
             site_url    => $blog->site_url
         };
-
-        # we should use count by group here...
-        $row->{num_entries} =
-          ( $entry_count ? $entry_count->{$blog_id} : $entry_count->{$blog_id} =
-              MT::Entry->count( { blog_id => $blog_id } ) )
-          || 0;
-        $row->{num_comments} = (
-              $comment_count
-            ? $comment_count->{$blog_id}
-            : $comment_count->{$blog_id} = MT::Comment->count(
-                { blog_id => $blog_id, junk_status => MT::Comment::NOT_JUNK() }
-            )
-          )
-          || 0;
-        $row->{num_pings} = (
-            $ping_count ? $ping_count->{$blog_id} : $ping_count->{$blog_id} =
-              MT::TBPing->count(
-                { blog_id => $blog_id, junk_status => MT::TBPing::NOT_JUNK() }
-              )
-        ) || 0;
-        $row->{num_authors} = 0;
-
-        # FIXME: This isn't efficient
-        my $iter = MT::Permission->load_iter(
-            {
-                blog_id => [ 0, $blog_id ],
-
-                #    role_mask => [ 2, undef ]
-                #}, {
-                #    range_incl => { 'role_mask' => 1 }
-            }
-        );
-        my %a;
-        while ( my $p = $iter->() ) {
-            next if exists $a{ $p->author_id };
-            $a{ $p->author_id } = 1;
-            $row->{num_authors}++ if $p->can_create_post;
-        }
-        if ( $author->is_superuser ) {
-            $row->{can_create_post}       = 1;
-            $row->{can_edit_entries}      = 1;
-            $row->{can_edit_templates}    = 1;
-            $row->{can_edit_config}       = 1;
-            $row->{can_set_publish_paths} = 1;
-            $row->{can_administer_blog}   = 1;
-        }
-        else {
-            my $perms = $author->permissions($blog_id);
-            $row->{can_create_post}  = $perms->can_create_post;
-            $row->{can_edit_entries} = $perms->can_create_post
-              || $perms->can_edit_all_posts
-              || $perms->can_publish_post;
-            $row->{can_edit_templates} = $perms->can_edit_templates;
-            $row->{can_edit_config}    = $perms->can_edit_config
-              || $perms->can_administer_blog;
-            $row->{can_set_publish_paths} = $perms->can_set_publish_paths
-              || $perms->can_administer_blog;
-            $row->{can_administer_blog} = $perms->can_administer_blog;
-        }
+        if ($app->mode ne 'dialog_select_weblog') {
+			# we should use count by group here...
+			$row->{num_entries} =
+			  ( $entry_count ? $entry_count->{$blog_id} : $entry_count->{$blog_id} =
+				  MT::Entry->count( { blog_id => $blog_id } ) )
+			  || 0;
+			$row->{num_comments} = (
+				  $comment_count
+				? $comment_count->{$blog_id}
+				: $comment_count->{$blog_id} = MT::Comment->count(
+					{ blog_id => $blog_id, junk_status => MT::Comment::NOT_JUNK() }
+				)
+			  )
+			  || 0;
+			$row->{num_pings} = (
+				$ping_count ? $ping_count->{$blog_id} : $ping_count->{$blog_id} =
+				  MT::TBPing->count(
+					{ blog_id => $blog_id, junk_status => MT::TBPing::NOT_JUNK() }
+				  )
+			) || 0;
+			if ( $author->is_superuser ) {
+				$row->{can_create_post}       = 1;
+				$row->{can_edit_entries}      = 1;
+				$row->{can_edit_templates}    = 1;
+				$row->{can_edit_config}       = 1;
+				$row->{can_set_publish_paths} = 1;
+				$row->{can_administer_blog}   = 1;
+			}
+			else {
+				my $perms = $author->permissions($blog_id);
+				$row->{can_create_post}  = $perms->can_create_post;
+				$row->{can_edit_entries} = $perms->can_create_post
+				  || $perms->can_edit_all_posts
+				  || $perms->can_publish_post;
+				$row->{can_edit_templates} = $perms->can_edit_templates;
+				$row->{can_edit_config}    = $perms->can_edit_config
+				  || $perms->can_administer_blog;
+				$row->{can_set_publish_paths} = $perms->can_set_publish_paths
+				  || $perms->can_administer_blog;
+				$row->{can_administer_blog} = $perms->can_administer_blog;
+			}
+		}
         $row->{object} = $blog;
         push @data, $row;
     }
