@@ -171,6 +171,23 @@ sub list {
     my $user  = $app->user;
     my $admin = $user->is_superuser
       || ( $perms && $perms->can_administer_blog );
+
+    unless ($app->user->is_superuser) {
+        if ( $app->param('blog_id') ) {
+            return $app->errtrans("Permission denied.")
+                unless $perms && $perms->can_view_feedback;
+        } else {
+            require MT::Permission;
+            my @blogs
+                = map { $_->blog_id }
+                grep {
+                    $_->can_view_feedback
+                } MT::Permission->load(
+                { author_id => $app->user->id } );
+            return $app->errtrans("Permission denied.") unless @blogs;
+        }
+    }
+
     my $can_empty_junk = $admin
       || ( $perms && $perms->can_manage_feedback )
       ? 1 : 0;
@@ -1684,6 +1701,7 @@ sub set_item_visible {
                         }
                         unless (
                             $perms->can_manage_feedback
+                            || $perms->can_edit_all_posts
                             || ( $perms->can_publish_post
                                 && ( $obj_parent->author_id == $author->id ) )
                           )
@@ -1713,6 +1731,7 @@ sub set_item_visible {
                     }
                     unless (
                         $perms->can_manage_feedback
+                        || $perms->can_edit_all_posts
                         || ( $perms->can_publish_post
                             && ( $entry->author_id == $author->id ) )
                       )
@@ -1844,7 +1863,7 @@ sub build_comment_table {
     my $title_max_len = const('DISPLAY_LENGTH_EDIT_COMMENT_TITLE');
 
     while ( my $obj = $iter->() ) {
-        my $row = $obj->column_values;
+        my $row = $obj->get_values;
         $row->{author_display} = $row->{author};
         $row->{author_display} =
           substr_text( $row->{author_display}, 0, $author_max_len ) . '...'
