@@ -62,6 +62,8 @@ sub dashboard {
                 File::Spec->catdir( $app->static_file_path, 'support' );
         }
     }
+    eval { require MT::Image; MT::Image->new or die; };
+    $param->{can_use_userpic} = $@ ? 0 : 1;
 
     # We require that the determination of the 'single blog mode'
     # state be done PRIOR to the generation of the widgets
@@ -122,6 +124,24 @@ sub this_is_you_widget {
         $param->{comment_count} = $count;
     }
 
+    require MT::Permission;
+    my @perm = MT::Permission->load(
+        { author_id => $app->user->id } );
+    my @blogs
+        = map { $_->blog_id }
+        grep {
+        $_->can_create_post
+            || $_->can_publish_post
+            || $_->can_edit_all_posts
+        } @perm;
+    $param->{can_list_entries} = @blogs ? 1: 0;
+    @blogs
+        = map { $_->blog_id }
+        grep {
+            $_->can_view_feedback
+        } @perm;
+    $param->{can_list_comments} = @blogs ? 1 : 0;
+
     my $last_post = MT::Entry->load(
         {
             author_id => $user->id,
@@ -136,8 +156,12 @@ sub this_is_you_widget {
     if ($last_post) {
         $param->{last_post_id}      = $last_post->id;
         $param->{last_post_blog_id} = $last_post->blog_id;
-        $param->{last_post_blog_name} = encode_html($last_post->blog->name);
+        $param->{last_post_blog_name} = $last_post->blog->name;
         $param->{last_post_ts}      = $last_post->authored_on;
+        my $perms = MT::Permission->load( 
+            { blog_id => $last_post->blog_id, author_id => $app->user->id } );
+        $param->{last_post_can_edit}
+            = $perms && $perms->can_edit_entry($last_post, $app->user);
     }
 
     if (my ($url) = $user->userpic_url()) {
