@@ -159,7 +159,11 @@ sub core_search_apis {
             'can_search_by_date' => 1,
             'setup_terms_args'   => sub {
                 my ($terms, $args, $blog_id) = @_;
-                $terms->{class} = '*';
+                $terms->{class}
+                    = ( $app->param('filter') 
+                        && $app->param('filter_val')
+                        && $app->param('filter') eq 'class'
+                        && $app->param('filter_val') eq 'image' ) ? 'image' : '*';
                 $terms->{blog_id} = $blog_id if $blog_id;
             }
         },
@@ -196,7 +200,7 @@ sub core_search_apis {
                 return 1 if $author->is_superuser;
                 if ($blog_id) {
                     my $perm = $author->permissions($blog_id);
-                    return $perm->can_administer_blog;
+                    return $perm->can_administer_blog || $perm->can_manage_users;
                 }
                 return 0;
             },
@@ -235,8 +239,7 @@ sub core_search_apis {
                 return 1 if $author->is_superuser;
                 my ($obj) = @_;
                 my $perm = $author->permissions( $obj->id );
-                $perm
-                  && ( $perm->can_administer_blog || $perm->can_edit_config );
+                return $perm && ( $perm->blog_id == $obj->id ) ? 1 : 0;
             },
             'search_cols' => {
                 'name' => sub { $app->translate('Name') },
@@ -384,6 +387,7 @@ sub do_search_replace {
     ## Sometimes we need to pass in the search columns like 'title,text', so
     ## we look for a comma (not a valid character in a column name) and split
     ## on it if it's there.
+    my $plain_search = $search;
     if ( ($search || '') ne '' ) {
         $search = quotemeta($search) unless $is_regex;
         $search = '(?i)' . $search   unless $case;
@@ -392,7 +396,14 @@ sub do_search_replace {
     my $api   = $search_api->{$type};
     my $class = $app->model($api->{object_type} || $type);
     my %param = %$list_pref;
-    my $limit = $q->param('limit') || 125;    # FIXME: mt.cfg setting?
+    my $limit;
+    # type-specific directives override global CMSSearchLimit
+    my $directive = 'CMSSearchLimit' . ucfirst($type);
+	$limit = MT->config->$directive || MT->config->CMSSearchLimit || 125;
+	# don't allow passed limit to be higher than config limit
+	if ($q->param('limit') && ($q->param('limit') < $limit)) {
+		$limit = $q->param('limit');
+	}
     $limit =~ s/\D//g;
     my $matches;
     $date_col = $api->{date_column} || 'created_on';
@@ -405,7 +416,7 @@ sub do_search_replace {
             if ($blog_id) {
                 my $perm = $author->permissions($blog_id);
                 return $app->errtrans('Permission denied.')
-                    unless $perm->can_administer_blog;
+                    unless $perm->can_administer_blog || $perm->can_manage_users;
             }
             $blog_id = 0;
         }
@@ -439,6 +450,22 @@ sub do_search_replace {
                   [ $datefrom . '000000', $dateto . '235959' ];
             }
         }
+        my @terms;
+        # MT::Object doesn't like multi-term hashes within arrays
+        if (%terms) {
+        	for my $key (keys %terms) {
+        		push(@terms, { $key => $terms{$key} });
+        	}
+        	push(@terms, '-and');
+        }
+        my @col_terms;
+        my $query_string = "%$plain_search%";
+        for my $col (@cols) {
+			push(@col_terms, { $col => { like => $query_string } }, '-or' );
+        }
+        delete $col_terms[$#col_terms];
+        push(@terms, \@col_terms);
+        $args{limit} = $limit + 1;
         my $iter;
         if ($do_replace) {
             $iter = sub {
@@ -451,13 +478,13 @@ sub do_search_replace {
               || ( $type eq 'blog' )
               || ( $app->mode eq 'dialog_grant_role' ) )
             {
-                $iter = $class->load_iter( \%terms, \%args ) or die $class->errstr;
+                $iter = $class->load_iter( \@terms, \%args ) or die $class->errstr;
             }
             else {
 
                 my @streams;
                 if ( $author->is_superuser ) {
-                    @streams = ( { iter => $class->load_iter( \%terms, \%args ) } );
+                    @streams = ( { iter => $class->load_iter( \@terms, \%args ) } );
                 } 
                 else {
                     # Get an iter for each accessible blog
@@ -467,12 +494,10 @@ sub do_search_replace {
                     );
                     if (@perms) {
                         @streams = map {
+                            $terms[0]{blog_id} = $_->blog_id;
                             {
                                 iter => $class->load_iter(
-                                    {
-                                        blog_id => $_->blog_id,
-                                        %terms
-                                    },
+                                    \@terms,
                                     \%args
                                 )
                             }
@@ -636,7 +661,9 @@ sub do_search_replace {
         }
         $param{object_type} = $type;
         if ( exists $api->{results_table_template} ) {
-            $param{results_template} = $api->{results_table_template};
+            my $tmpl = _load_template( $app, $api->{results_table_template},
+                exists( $api->{plugin} ) ? $api->{plugin} : undef );
+            $param{results_template} = $tmpl;
         }
         else {
             $param{results_template} = _default_results_table_template($app, $type, 1, $class->class_label_plural);
@@ -644,7 +671,9 @@ sub do_search_replace {
     }
     else {
         if ( exists $api->{no_results_template} ) {
-            $param{results_template} = $api->{no_results_template};
+            my $tmpl = _load_template( $app, $api->{no_results_template},
+                exists( $api->{plugin} ) ? $api->{plugin} : undef );
+            $param{results_template} = $tmpl;
         }
         else {
             $param{results_template} = _default_results_table_template($app, $type, 0, $class->class_label_plural);
@@ -732,4 +761,35 @@ TMPL
     }
 }
 
+sub _load_template {
+    my $app = shift;
+    my ( $tmpl, $plugin ) = @_;
+
+    my $meth;
+    if ( ref $tmpl eq 'HASH' ) {
+        $meth = $tmpl->{handler} || $tmpl->{code};
+    }
+    elsif ( $tmpl =~ m/^\$(\w+)::/ ) {
+        $meth = $tmpl;
+    }
+    $tmpl = $app->handler_to_coderef($meth) if $meth;
+    if ( ref $tmpl eq 'CODE' ) {
+        $tmpl = $tmpl->($plugin || $app, @_);
+    }
+    if ($tmpl !~ /\s/) {
+        # no spaces in $tmpl; must be a filename...
+        my $tmpl_obj;
+        if ( $plugin ) {
+            $tmpl_obj = $plugin->load_tmpl($tmpl);
+        }
+        else {
+            $tmpl_obj = $app->load_tmpl($tmpl);
+        }
+        if ( $tmpl_obj ) {
+            $tmpl = $tmpl_obj->text;
+        }
+    }
+    return $tmpl;
+}
+
 1;
