@@ -4,7 +4,7 @@ use strict;
 use Symbol;
 
 use MT::I18N qw( encode_text wrap_text );
-use MT::Util qw( encode_url encode_html decode_html encode_js );
+use MT::Util qw( encode_url encode_html decode_html encode_js trim );
 
 sub system_check {
     my $app = shift;
@@ -32,25 +32,26 @@ sub system_check {
     $param{user_count} = $author_class->count(
         { type => MT::Author::AUTHOR() } );
 
-    # commeters: users with only comment permission and MT::Author::COMMENTER
-    my $cmntrs = $author_class->count(
-        { type => MT::Author::COMMENTER() } );
-
-    my @perms = $app->model('permission')->load(
-      {
-        permissions => "%'comment'%", 
-        blog_id     => '0',
-      },
-      {
-        'like' => { 'permissions' => 1 },
-        'not'  => { 'blog_id'     => 1 },
-      }
-    );
-    @perms = grep { $_->permissions =~ m/'comment'/ } @perms;
-    $param{commenter_count} = scalar(@perms) + $cmntrs;
+    $param{commenter_count} = q[N/A];
     $param{screen_id} = "system-check";
+
+    require MT::Memcached;
+    if (MT::Memcached->is_available) {
+        $param{memcached_enabled} = 1;
+        my $inst = MT::Memcached->instance;
+        my $key = 'syscheck-' . $$;
+        $inst->add($key, $$);
+        if ($inst->get($key) == $$) {
+            $inst->delete($key);
+            $param{memcached_active} = 1;
+        }
+    }
+
+    $param{server_modperl} = 1 if $ENV{MOD_PERL};
+    $param{server_fastcgi} = 1 if $ENV{FAST_CGI};
+
     $param{syscheck_html} = get_syscheck_content($app) || '';
-    
+
     $app->load_tmpl( 'system_check.tmpl', \%param );
 }
 
@@ -106,39 +107,219 @@ sub get_syscheck_content {
 
 sub start_recover {
     my $app = shift;
+    my ($param) = @_;
+    my $cfg = $app->config;
+    $param ||= {};
+    $param->{'email'} = $app->param('email');
+    $param->{'return_to'} = $app->param('return_to') || $cfg->ReturnToURL || '';
+    $param->{'can_signin'} = (ref $app eq 'MT::App::CMS') ? 1 : 0;
     $app->add_breadcrumb( $app->translate('Password Recovery') );
-    $app->load_tmpl('dialog/recover.tmpl');
+
+    my $blog_id = $app->param('blog_id');
+    $param->{'blog_id'} = $blog_id;
+    my $tmpl = $app->load_global_tmpl( { identifier => 'new_password_reset_form',
+            $blog_id ? ( blog_id => $app->param('blog_id') ) : () } );
+    if (!$tmpl) {
+        $tmpl = $app->load_tmpl( 'cms/dialog/recover.tmpl' );
+    }
+    $tmpl->param($param);
+    return $tmpl;
 }
 
 sub recover_password {
-    my $app   = shift;
-    my $q     = $app->param;
-    my $name  = $q->param('name');
-    my $class = ref $app eq 'MT::App::Upgrader' ? 'MT::BasicAuthor' : $app->model('author');
-    eval "use $class;";
-    my @author = $class->load( { name => $name } );
-    my $author;
-    foreach (@author) {
-        next unless $_->password && ( $_->password ne '(none)' );
-        $author = $_;
+    my $app      = shift;
+    my $email    = $app->param('email') || '';
+    my $username = $app->param('name');
+
+    $email = trim($email);
+    $username = trim($username) if $username;
+
+    if ( !$email ) {
+        return $app->start_recover(
+            { error => $app->translate('Email Address is required for password recovery.'), } );
     }
 
-    my ( $rc, $res ) =
-      reset_password( $app, $author, $q->param('hint'), $name );
+    # Searching user by email (and username)
+    my $class
+        = ref $app eq 'MT::App::Upgrader'
+        ? 'MT::BasicAuthor'
+        : $app->model('author');
+    eval "use $class;";
 
-    if ($rc) {
-        $app->add_breadcrumb( $app->translate('Password Recovery') );
-        $app->load_tmpl(
-            'dialog/recover.tmpl',
-            {
-                recovered => 1,
-                email     => $author->email
+    my @all_authors = $class->load(
+        { email => $email, ( $username ? ( name => $username ) : () ) } );
+    my @authors;
+    my $user;
+    foreach (@all_authors) {
+        next unless $_->password && ( $_->password ne '(none)' );
+        push( @authors, $_ );
+    }
+    if ( !@authors ) {
+        return $app->start_recover(
+            {   error => $app->translate('User not found'),
+                ( $username ? ( not_unique_email => 1 ) : () ),
             }
         );
     }
-    else {
-        $app->error($res);
+    elsif ( @authors > 1 ) {
+        return $app->start_recover( { not_unique_email => 1, } );
     }
+    $user = pop @authors;
+
+    # Generate Token
+    require MT::Util::Captcha;
+    my $salt    = MT::Util::Captcha->_generate_code(8);
+    my $expires = time + ( 60 * 60 );
+    my $token = MT::Util::perl_sha1_digest_hex(
+        $salt . $expires . $app->config->SecretToken );
+
+    $user->password_reset($salt);
+    $user->password_reset_expires($expires);
+    $user->password_reset_return_to($app->param('return_to'))
+        if $app->param('return_to');
+    $user->save;
+
+    # Send mail to user
+    my %head = (
+        id      => 'recover_password',
+        To      => $email,
+        From    => $app->config('EmailAddressMain') || $email,
+        Subject => $app->translate("Password Recovery")
+    );
+    my $charset = $app->charset;
+    my $mail_enc = uc( $app->config('MailEncoding') || $charset );
+    $head{'Content-Type'} = qq(text/plain; charset="$mail_enc");
+
+    my $blog_id = $app->param('blog_id');
+    my $body = $app->build_email(
+        'recover-password',
+        {         link_to_login => $app->base
+                . $app->uri
+                . "?__mode=new_pw&token=$token&email="
+                . encode_url($email)
+                . ($blog_id ? "&blog_id=$blog_id" : ''),
+        }
+    );
+
+    require MT::Mail;
+    MT::Mail->send( \%head, $body )
+        or return $app->error(
+        $app->translate(
+            "Error sending mail ([_1]); please fix the problem, then "
+                . "try again to recover your password.",
+            MT::Mail->errstr
+        )
+        );
+
+    return $app->start_recover( { recovered => 1, } );
+}
+
+sub new_password {
+    my $app = shift;
+    my ($param) = @_;
+    $param ||= {};
+
+    my $token = $app->param('token');
+    if ( !$token ) {
+        return $app->start_recover(
+            { error => $app->translate('Password reset token not found'), } );
+    }
+
+    my $email = $app->param('email');
+    if ( !$email ) {
+        return $app->start_recover(
+            { error => $app->translate('Email address not found'), } );
+    }
+
+    my $class = $app->model('author');
+    my @users = $class->load( { email => $email } );
+    if ( !@users ) {
+        return $app->start_recover(
+            { error => $app->translate('User not found'), } );
+    }
+
+    # comparing token
+    require MT::Util::Captcha;
+    my $user;
+    for my $u (@users) {
+        my $salt    = $u->password_reset;
+        my $expires = $u->password_reset_expires;
+        my $compare = MT::Util::perl_sha1_digest_hex(
+            $salt . $expires . $app->config->SecretToken );
+        if ( $compare eq $token ) {
+            if ( time > $u->password_reset_expires ) {
+                return $app->start_recover(
+                    {   error => $app->translate(
+                            'Your request to change your password has expired.'
+                        ),
+                    }
+                );
+            }
+            $user  = $u;
+            last;
+        }
+    }
+
+    if ( !$user ) {
+        return $app->start_recover(
+            { error => $app->translate('Invalid password reset request'), } );
+    }
+
+    # Password reset
+    my $new_password = $app->param('password');
+    if ($new_password) {
+        my $again = $app->param('password_again');
+        if ( !$again ) {
+            $param->{'error'}
+                = $app->translate('Please confirm your new password');
+        }
+        elsif ( $new_password ne $again ) {
+            $param->{'error'} = $app->translate('Passwords do not match');
+        }
+        else {
+            my $redirect = $user->password_reset_return_to || '';
+
+            $user->set_password($new_password);
+            $user->password_reset(undef);
+            $user->password_reset_expires(undef);
+            $user->password_reset_return_to(undef);
+            $user->save;
+            $app->param( 'username', $user->name )
+                if $user->type == MT::Author::AUTHOR();
+
+            if (ref $app eq 'MT::App::CMS' && !$redirect) {
+                $app->login;
+                return $app->return_to_dashboard( redirect => 1 );
+            } else {
+                if (!$redirect) {
+                    my $cfg = $app->config;
+                    $redirect = $cfg->ReturnToURL || '';
+                }
+                $app->make_commenter_session($user);
+                if ($redirect) {
+                    return $app->redirect($redirect);
+                } else {
+                    return $app->redirect_to_edit_profile();
+                }
+            }
+        }
+    }
+
+    $param->{'email'}          = $email;
+    $param->{'token'}          = $token;
+    $param->{'password'}       = $app->param('password');
+    $param->{'password_again'} = $app->param('password_again');
+    $app->add_breadcrumb( $app->translate('Password Recovery') );
+
+    my $blog_id = $app->param('blog_id');
+    $param->{'blog_id'}        = $blog_id if $blog_id;
+    my $tmpl = $app->load_global_tmpl( { identifier => 'new_password',
+            $blog_id ? ( blog_id => $app->param('blog_id') ) : () } );
+    if (!$tmpl) {
+        $tmpl = $app->load_tmpl( 'cms/dialog/new_password.tmpl' );
+    }
+    $tmpl->param($param);
+    return $tmpl;
 }
 
 sub do_list_action {
@@ -928,7 +1109,7 @@ sub restore_premature_cancel {
     $app->validate_magic() or return;
 
     require JSON;
-    my $deferred = JSON::jsonToObj( $app->param('deferred_json') )
+    my $deferred = JSON::from_json( $app->param('deferred_json') )
       if $app->param('deferred_json');
     my $param = { restore_success => 1 };
     if ( defined $deferred && ( scalar( keys %$deferred ) ) ) {
@@ -1142,8 +1323,7 @@ sub adjust_sitepath {
     my $param = {};
     if ( scalar $q->param('redirect') ) {
         $param->{restore_end} = 0;  # redirect=1 means we are from multi-uploads
-        require JSON;
-        $param->{blogs_meta} = JSON::objToJson( \%blogs_meta );
+        $param->{blogs_meta} = MT::Util::to_json( \%blogs_meta );
         $param->{next_mode}  = 'dialog_restore_upload';
     }
     else {
@@ -1184,7 +1364,7 @@ sub dialog_restore_upload {
     my $deferred = {};
     require JSON;
     my $objects_json = $q->param('objects_json') if $q->param('objects_json');
-    $deferred = JSON::jsonToObj( $q->param('deferred_json') )
+    $deferred = JSON::from_json( $q->param('deferred_json') )
       if $q->param('deferred_json');
 
     my ($fh) = $app->upload_info('file');
@@ -1199,7 +1379,7 @@ sub dialog_restore_upload {
     $param->{redirect}       = 1;
     $param->{is_dirty}       = $q->param('is_dirty');
     $param->{objects_json}   = $objects_json if defined($objects_json);
-    $param->{deferred_json}  = JSON::objToJson($deferred) if defined($deferred);
+    $param->{deferred_json}  = MT::Util::to_json($deferred) if defined($deferred);
     $param->{blogs_meta}     = $q->param('blogs_meta');
     $param->{schema_version} = $schema_version;
     $param->{overwrite_templates} = $overwrite_template;
@@ -1231,7 +1411,7 @@ sub dialog_restore_upload {
     $app->print( $app->build_page( 'dialog/restore_start.tmpl', $param ) );
 
     if ( defined $objects_json ) {
-        my $objects_tmp = JSON::jsonToObj($objects_json);
+        my $objects_tmp = JSON::from_json($objects_json);
         my %class2ids;
 
         # { MT::CLASS#OLD_ID => NEW_ID }
@@ -1266,7 +1446,7 @@ sub dialog_restore_upload {
         }
     }
 
-    my $assets = JSON::jsonToObj( decode_html($assets_json) )
+    my $assets = JSON::from_json( decode_html($assets_json) )
       if ( defined($assets_json) && $assets_json );
     $assets = [] if !defined($assets);
     my $asset;
@@ -1279,7 +1459,7 @@ sub dialog_restore_upload {
     if ($is_asset) {
         $asset = shift @$assets;
         $asset->{fh} = $fh;
-        my $blogs_meta = JSON::jsonToObj( $q->param('blogs_meta') || '{}' );
+        my $blogs_meta = JSON::from_json( $q->param('blogs_meta') || '{}' );
         MT::BackupRestore->_restore_asset_multi( $asset, $objects,
             $error_assets, sub { $app->print(@_); }, $blogs_meta );
         if ( defined( $error_assets->{ $asset->{asset_id} } ) ) {
@@ -1315,7 +1495,7 @@ sub dialog_restore_upload {
         }
     }
     $param->{files}  = join( ',', @files );
-    $param->{assets} = encode_html( JSON::objToJson($assets) );
+    $param->{assets} = encode_html( MT::Util::to_json($assets) );
     $param->{name}   = $file_next;
     if ( 0 < scalar(@files) ) {
         $param->{last} = 0;
@@ -1367,8 +1547,8 @@ sub dialog_restore_upload {
     else {
         my %objects_json;
         %objects_json = map { $_ => $objects->{$_}->id } keys %$objects;
-        $param->{objects_json}  = JSON::objToJson( \%objects_json );
-        $param->{deferred_json} = JSON::objToJson($deferred);
+        $param->{objects_json}  = MT::Util::to_json( \%objects_json );
+        $param->{deferred_json} = MT::Util::to_json($deferred);
 
         $param->{error} = join( '; ', @errors );
         if ( defined($blog_ids) && scalar(@$blog_ids) ) {
@@ -1530,14 +1710,21 @@ sub reset_password {
     return ( 0, $app->translate("User does not have email address") )
       unless $author->email;
 
-    my @pool = ( 'a' .. 'z', 0 .. 9 );
-    my $pass = '';
-    for ( 1 .. 8 ) { $pass .= $pool[ rand @pool ] }
-    $author->set_password($pass);
+    # Generate Token
+    require MT::Util::Captcha;
+    my $salt    = MT::Util::Captcha->_generate_code(8);
+    my $expires = time + ( 60 * 60 );
+    my $token = MT::Util::perl_sha1_digest_hex(
+        $salt . $expires . $app->config->SecretToken );
+
+    $author->password_reset($salt);
+    $author->password_reset_expires($expires);
+    $author->password_reset_return_to(undef);
     $author->save;
+
     my $message =
       $app->translate(
-"Password was reset for user '[_1]' (user #[_2]). Password was sent to the following address: [_3]",
+"A password reset link has been sent to [_3] for user  '[_1]' (user #[_2]).",
         $author->name, $author->id, $author->email );
     $app->log(
         {
@@ -1548,34 +1735,37 @@ sub reset_password {
         }
     );
 
-    my $address =
-      defined $author->nickname
-      ? $author->nickname . ' <' . $author->email . '>'
-      : $author->email;
+    # Send mail to user
+    my $email = $author->email;
     my %head = (
         id      => 'recover_password',
-        To      => $address,
-        From    => $app->config('EmailAddressMain') || $address,
+        To      => $email,
+        From    => $app->config('EmailAddressMain') || $email,
         Subject => $app->translate("Password Recovery")
     );
     my $charset = $app->charset;
     my $mail_enc = uc( $app->config('MailEncoding') || $charset );
     $head{'Content-Type'} = qq(text/plain; charset="$mail_enc");
 
-    my $body = $app->build_email( 'recover-password.tmpl',
-        { user_password => $pass, link_to_login => $app->base . $app->mt_uri } 
+    my $body = $app->build_email(
+        'recover-password',
+        {         link_to_login => $app->base
+                . $app->uri
+                . "?__mode=new_pw&token=$token&email="
+                . encode_url($email),
+        }
     );
-    $body = wrap_text( $body, 72 );
+
     require MT::Mail;
     MT::Mail->send( \%head, $body )
-      or return (
-        0,
+        or return $app->error(
         $app->translate(
             "Error sending mail ([_1]); please fix the problem, then "
-              . "try again to recover your password.",
+                . "try again to recover your password.",
             MT::Mail->errstr
         )
-      );
+        );
+
     ( 1, $message );
 }
 
@@ -1744,8 +1934,7 @@ sub restore_upload_manifest {
             $param->{is_asset} = 1;
         }
     }
-    require JSON;
-    $assets_json = encode_url( JSON::objToJson($assets) )
+    $assets_json = encode_url( MT::Util::to_json($assets) )
       if scalar(@$assets) > 0;
     $param->{files}       = join( ',', @$files );
     $param->{assets}      = $assets_json;
