@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Negotiate.pm 1513 2008-03-12 08:25:59Z fumiakiy $
+# $Id: Negotiate.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::Cache::Negotiate;
 
@@ -43,7 +43,9 @@ in memcached or in MT::Session table.
 
 =head1 SYNOPSIS
 
-    my $cache = MT::Cache::Negotiate->new({ttl => 10});
+    # MT::Cache::Session uses the 'kind' parameter
+    # which accepts namespace of the cache
+    my $cache = MT::Cache::Negotiate->new( ttl => 10, kind => 'XX' );
     my $data = $cache->get($key);
     $cache->set($key => $value);
     my $hash = $cache->get_multi($key1, $key2);
