@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Component.pm 2888 2008-08-02 23:02:16Z bchoate $
+# $Id: Component.pm 3473 2009-02-27 20:23:25Z bchoate $
 
 package MT::Component;
 
@@ -50,9 +50,6 @@ sub new {
 sub init {
     my $c = shift;
     $c->init_registry() or return;
-
-    # plugin callbacks are initialized after they finish loading.
-    $c->init_callbacks() unless $c->isa('MT::Plugin');
     $c;
 }
 
@@ -460,8 +457,7 @@ sub translate_templatized {
             '';
         } else {
             $args{params} = '' unless defined $args{params};
-            my @p = map MT::Util::decode_html($_),
-                    split /\s*%%\s*/, $args{params}, -1;
+            my @p = split /\s*%%\s*/, $args{params}, -1;
             @p = ('') unless @p;
             my $translation = $c->translate($args{phrase}, @p);
             if (exists $args{escape}) {
