@@ -1,4 +1,4 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -9,7 +9,7 @@ package MT::ConfigMgr;
 use strict;
 use base qw( MT::ErrorHandler );
 
-use vars qw( $cfg );
+our $cfg;
 sub instance {
     return $cfg if $cfg;
     $cfg = __PACKAGE__->new;
@@ -104,7 +104,8 @@ sub get {
 sub type {
     my $mgr = shift;
     my $var = lc shift;
-    $mgr->{__settings}{$var}{type} || 'SCALAR';
+    return undef unless exists $mgr->{__settings}{$var};
+    return $mgr->{__settings}{$var}{type} || 'SCALAR';
 }
 
 sub default {
@@ -129,8 +130,7 @@ sub set_internal {
     my $mgr = shift;
     my($var, $val, $db_flag) = @_;
     $var = lc $var;
-    $db_flag ||= exists $mgr->{__dbvar}{$var};
-    $mgr->set_dirty() if defined($_[2]) && $_[2];
+    $mgr->set_dirty() if defined($db_flag) && $db_flag;
     my $set = $db_flag ? '__dbvar' : '__var';
     if (defined(my $alias = $mgr->{__settings}{$var}{alias})) {
         if ($max_depth < $depth) {
@@ -169,6 +169,7 @@ sub set {
     my $mgr = shift;
     my($var, $val, $db_flag) = @_;
     $var = lc $var;
+    $mgr->set_dirty($var);
     if (my $h = $mgr->{__settings}{$var}{handler}) {
         $h = MT->handler_to_coderef($h) unless ref $h;
         return $h->($mgr, $val, $db_flag);
@@ -179,30 +180,40 @@ sub set {
 sub is_readonly {
     my $class = shift;
     my ($var) = @_;
-    defined $class->instance->{__var}{lc $var} ? 1 : 0;
+    return defined $class->instance->{__var}{lc $var} ? 1 : 0;
 }
 
 sub read_config {
     my $class = shift;
-    $class->read_config_file(@_);
+    return $class->read_config_file(@_);
 }
 
 sub set_dirty {
     my $mgr = shift;
+    my ($var) = @_;
     $mgr = $mgr->instance unless ref($mgr);
-    $mgr->{__dirty} = 1;
+    return $mgr->{__settings}{lc $var}{dirty} = 1 if defined $var;
+    return $mgr->{__dirty} = 1;
 }
 
 sub clear_dirty {
     my $mgr = shift;
+    my ($var) = @_;
     $mgr = $mgr->instance unless ref($mgr);
-    $mgr->{__dirty} = 0;
+    return delete $mgr->{__settings}{lc $var}{dirty} if defined $var;
+    foreach my $var ( keys %{ $mgr->{__settings}} ) {
+        if ( $mgr->{__settings}{$var}{dirty} ) {
+            delete $mgr->{__settings}{$var}{dirty};
+        }
+    }
+    return $mgr->{__dirty} = 0;
 }
 
 sub is_dirty {
     my $mgr = shift;
     $mgr = $mgr->instance unless ref($mgr);
-    $mgr->{__dirty};
+    return $mgr->{__settings}{lc $_[0]}{dirty} ? 1 : 0 if @_;
+    return $mgr->{__dirty};
 }
 
 sub save_config {
@@ -215,6 +226,8 @@ sub save_config {
     my $settings = $mgr->{__dbvar};
     foreach (sort keys %$settings) {
         my $type = ($mgr->{__settings}{$_}{type}||'');
+        delete $mgr->{__settings}{$_}{dirty}
+            if exists $mgr->{__settings}{$_}{dirty};
         if ($type eq 'HASH') {
             my $h = $settings->{$_};
             foreach my $k (keys %$h) {
@@ -229,8 +242,10 @@ sub save_config {
             $data .= $mgr->{__settings}{$_}{key} . ' ' . $settings->{$_} . "\n";
         }
     }
-    require MT::Config;
-    my ($config) = MT::Config->load() || new MT::Config;
+
+    my $cfg_class = MT->model('config') or return;
+
+    my ($config) = $cfg_class->load() || $cfg_class->new;
 
     if ($data !~ m/schemaversion/i) {
         if ($config->id && (($config->data || '') =~ m/schemaversion/i)) {
@@ -268,10 +283,6 @@ sub read_config_file {
             unless defined($val) && $val ne '';
         $val =~ s/\s*$// if defined($val);
         next unless $var && defined($val);
-        #return $class->error(MT->translate(
-        #    "[_1]:[_2]: variable '[_3]' not defined", $cfg_file, $., $var
-        #    )) unless exists $mgr->{__settings}->{$var};
-        # next unless exists $mgr->{__settings}->{$var};
         $mgr->set($var, $val);
     }
     close FH;
@@ -281,9 +292,11 @@ sub read_config_file {
 sub read_config_db {
     my $class = shift;
     my $mgr = $class->instance;
-    require MT::Config;
-    my ($config) = eval { MT::Config->search };
+    my $cfg_class = MT->model('config') or return;
+
+    my ($config) = eval { $cfg_class->search };
     if ($config) {
+        my $was_dirty = $mgr->is_dirty;
         my $data = $config->data;
         my @data = split /[\r?\n]/, $data;
         my $line = 0;
@@ -294,20 +307,19 @@ sub read_config_db {
             my($var, $val) = $_ =~ /^\s*(\S+)\s+(.+)$/;
             $val =~ s/\s*$// if defined($val);
             next unless $var && defined($val);
-            #return $class->error(MT->translate(
-            #    "[_1]:[_2]: variable '[_3]' not defined", "database", $line, $var
-            #)) unless exists $mgr->{__settings}->{$var};
-
-            # ignore setting if it isn't defined...
-            # next unless exists $mgr->{__settings}->{$var};
             $mgr->set($var, $val, 1);
         }
+        $mgr->clear_dirty unless $was_dirty;
     }
     $mgr->{__read_db} = 1;
     1;
 }
 
-sub DESTROY { }
+sub DESTROY {
+    # save_config here so not to miss any dirty config change to persist
+    # particularly for those which does not construct MT::App.
+    $_[0]->save_config unless UNIVERSAL::isa( MT->instance, 'MT::App' );
+}
 
 use vars qw( $AUTOLOAD );
 sub AUTOLOAD {
@@ -341,52 +353,161 @@ MT::ConfigMgr - Movable Type configuration manager
 
 =head1 DESCRIPTION
 
-I<MT::ConfigMgr> is a singleton class that manages the Movable Type
-configuration file (F<mt.cfg>), allowing access to the config directives
-contained therin.
+L<MT::ConfigMgr> is a singleton class that manages the Movable Type
+configuration file (F<mt-config.cgi>), allowing access to the config
+directives contained therin.
 
 =head1 USAGE
 
+=head2 MT::ConfigMgr->new
+
+Creates a new instance of L<MT::ConfigMgr> and initializes it. It does not
+read any configuration file data. This is done using the L<read_config>
+method.
+
+=head2 $cfg->init
+
+Initialization method called by the L<new> constructor prior to returning
+a new instance of L<MT::ConfigMgr>.
+
 =head2 MT::ConfigMgr->instance
 
-Returns the singleton I<MT::ConfigMgr> object. Note that when you want the
-object, you should always call I<instance>, never I<new>; I<new> will construct
-a B<new> I<MT::ConfigMgr> object, and that isn't what you want. You want the
-object that has already been initialized with the contents of F<mt.cfg>. This
-initialization is done by I<MT::new>.
+Returns the singleton L<MT::ConfigMgr> object. Note that when you want
+the object, you should always call L<instance>, never L<new>; L<new>
+will construct a B<new> L<MT::ConfigMgr> object, and that isn't what you
+want. You want the object that has already been initialized with the
+contents of the configuration file. This initialization is done
+by L<MT::new>.
 
 =head2 $cfg->read_config($file)
 
+Calls L<read_config_file>.
+
+=head2 $cfg->save_config()
+
+Saves any configuration settings that originated from the database, or
+were set with the I<persist> option. Settings are stored using the
+L<MT::Config> class.
+
+=head2 $cfg->read_config_file($file)
+
 Reads the config file at the path I<$file> and initializes the I<$cfg> object
 with the directives in that file. Returns true on success, C<undef> otherwise;
 if an error occurs you can obtain the error message with C<$cfg-E<gt>errstr>.
 
-=head2 $cfg->define($directive [, %arg ])
+=head2 $cfg->read_config_db()
 
-Defines the directive I<$directive> as a valid configuration directive; you
-must define new configuration directives B<before> you read the configuration
-file, or else the read will fail.
+Reads any configuration settings from the L<MT::Config> class. Note that
+these settings are always overridden by settings in the MT configuration
+file.
 
-=head2 $cfg->ExternalUserManagement()
+=head2 $cfg->define($directive[, %arg ])
 
-Returns boolean value indicating whether the configuration is set so that
-external users management feature in Movable Type Enterprise is turned on.
+Defines the directive I<$directive> as a valid configuration directive. For
+special configuration directives (HASH or ARRAY types), you must define them
+B<before> you the configuration file is read.
+
+=head2 $cfg->set($directive, $value[, $persist])
+
+The handler method for assigning a value to a specific directive. The
+C<$value> should be a SCALAR value for simple configuration settings.
+
+    $cfg->set('EmailAddressMain', 'user@example.com');
+
+For an ARRAY type, C<$value> should be an array reference; if it is a
+SCALAR value, then it is added to any existing array held for the
+directive.
+
+    # Replaces any existing array value for 'MemcachedServers':
+    $cfg->set('MemcachedServers', ['127.0.0.1', '127.0.0.2']);
+
+    # Adds '127.0.0.3' to the existing array held for 'MemcachedServers':
+    $cfg->set('MemcachedServers', '127.0.0.3');
+
+For a HASH type, C<$value> should be a hash reference; if it is a SCALAR
+value, it should be in the format "key=value", and will be added any
+existing hash held for the directive.
+
+    # Replaces any existing hash value for 'AtomApp':
+    $cfg->set('AtomApp', { pings => 'Example::AtomPingServer' });
+
+    # Adds a new service declaration to the existing hash held for 'AtomApp':
+    $cfg->set('AtomApp', 'foo=Example::Foo');
+
+=head2 $cfg->paths
+
+Returns a list or array reference (depending on whether it is called in
+an array or scalar context) of configuration directive names that are
+declared as path directives.
+
+=head2 $cfg->get($directive)
+
+=head2 $cfg->get_internal($directive)
+
+The low-level method for getting a configuration setting and bypasses any
+declared 'handler'.
+
+=head2 $cfg->set_internal($directive, $value[, $persist])
+
+The low-level method for setting a configuration setting that bypasses any
+declared 'handler' and prevents the directive from having its "dirty"
+state set.
+
+=head2 $cfg->type($directive)
+
+Returns the type of the configuration directive (returns 'SCALAR', 'ARRAY'
+or 'HASH'). If the directive is unregistered, this method will return
+undef.
+
+=head2 $cfg->is_readonly($directive)
+
+Returns true when there exists a user-defined value for the configuration
+directive that was read from the MT configuration file. Such a value cannot
+be overridden through the database, so it is considered a read-only
+setting.
+
+=head2 $cfg->set_dirty([$directive])
+
+Assigns a dirty state to the configuration settings as a whole, or to
+an individual directive. The former controls whether or not the
+L<save_config> method will actually rewrite the configuration settings
+to the L<MT::Config> object. The latter is used to identify when a
+setting has been set to something other than the default.
+
+=head2 $cfg->is_dirty([$directive])
+
+Returns the 'dirty' state of the configuration settings as a whole, or
+for an individual directive.
+
+=head2 $cfg->clear_dirty([$directive])
+
+Clears the 'dirty' state of the configuration settings as a whole, or
+for an individual directive.
+
+=head2 $cfg->default($directive)
+
+Returns the default setting for the specified directive, if one exists.
 
 =head1 CONFIGURATION DIRECTIVES
 
-The following configuration directives are allowed in F<mt.cfg>. To get the
-value of a directive, treat it as a method that you are calling on the
-I<$cfg> object. For example:
+Once the I<ConfigMgr> object has been constructed, you can use it to obtain
+the configuration settings. Any of the defined settings may be gathered
+using a dynamic method invoked directly from the object:
 
-    $cfg->CGIPath
+    my $path = $cfg->CGIPath
 
-To set the value of a directive, do the same as the above, but pass in a value
-to the method:
+To set the value of a directive, do the same as the above, but pass in a
+value to the method:
 
     $cfg->CGIPath('http://www.foo.com/mt/');
 
-A list of valid configuration directives can be found in the
-I<CONFIGURATION SETTINGS> section of the manual.
+If you wish to progammatically assign a configuration setting that will
+persist, add an extra parameter when doing an assignment, passing '1'
+(this second parameter is a boolean that will cause the value to persist,
+using the L<MT::Config> class to store the settings into the datatbase):
+
+    $cfg->EmailAddressMain('user@example.com', 1);
+    $cfg->save_config;
 
 =head1 AUTHOR & COPYRIGHT
 
