@@ -1,14 +1,14 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: DefaultTemplates.pm 3130 2008-10-24 00:53:02Z bchoate $
+# $Id: DefaultTemplates.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::DefaultTemplates;
 
 use strict;
 
-=start
+=pod
 Registry storage format for default templates:
 
 default_templates:
@@ -399,11 +399,24 @@ sub templates {
         }
     }
     my @tmpls = (values(%tmpls), values(%global_tmpls));
-    @tmpls = sort { $a->{order} <=> $b->{order} } @tmpls;
+    # sort widgets to process last, since they rely on the widgets to exist first.
+    @tmpls = sort _template_sort @tmpls;
     MT->run_callbacks('DefaultTemplateFilter' . ($set ? '.' . $set : ''), \@tmpls);
     return \@tmpls;
 }
 
+sub _template_sort {
+    if ( $a->{type} eq 'widgetset' ) {
+        return 1 unless $b->{type} eq 'widgetset';
+    }
+    elsif ($b->{type} eq 'widgetset') {
+        # a is not a widgetset
+        return -1;
+    }
+    # both a, b == widgetset or both a, b != widgetset
+    return $a->{order} <=> $b->{order};
+}
+
 1;
 __END__
 
