@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Object.pm 3092 2008-10-08 19:29:24Z bchoate $
+# $Id: Object.pm 3774 2009-06-01 21:52:50Z jmarcotte $
 
 package MT::Object;
 
@@ -136,6 +136,13 @@ sub install_properties {
         $type_id = $props->{datasource};
     }
 
+    $props->{get_driver} ||= sub {
+        require MT::ObjectDriverFactory;
+        my $coderef = MT::ObjectDriverFactory->driver_for_class($class);
+        $class->get_driver($coderef);
+        return $coderef->(@_);
+    };
+
     $class->SUPER::install_properties($props);
 
     # check for any supplemental columns from other components
@@ -225,12 +232,6 @@ sub install_properties {
         $class->add_trigger( post_load => _get_date_translator(\&_db2ts, 0) );
     }
 
-    if ( exists($props->{cacheable}) && !$props->{cacheable} ) {
-        no warnings 'redefine';
-        no strict 'refs'; ## no critic
-        *{$class . '::driver'} = sub { $_[0]->dbi_driver(@_) };
-    }
-
     # inherit parent's metadata setup
     if ($props->{meta}) { # if ($super_props && $super_props->{meta_installed}) {
         $class->install_meta({ ( %meta ? ( column_defs => \%meta ) : ( columns => [] ) ) });
@@ -556,9 +557,11 @@ sub has_column {
 
 sub _post_save_save_metadata {
     my $obj = shift;
+    my ($orig_obj) = @_;
     if (defined $obj && exists $obj->{__meta}) {
         $obj->{__meta}->set_primary_keys($obj);
         $obj->{__meta}->save;
+        $orig_obj->{__meta} = $obj->{__meta};
     }
 }
 
@@ -855,15 +858,18 @@ sub __properties { }
 
 our $DRIVER;
 sub driver {
-    require MT::ObjectDriverFactory;
-    return $DRIVER ||= MT::ObjectDriverFactory->new;
+    my $class = shift;
+    return $DRIVER ||= MT::ObjectDriverFactory->instance
+        if UNIVERSAL::isa($class, 'MT::Object');
+    my $driver = $class->SUPER::driver(@_);
+    return $driver;
 }
 
-# ref to the fallback driver for non-cacheable classes
-our $DBI_DRIVER;
 sub dbi_driver {
-    unless ($DBI_DRIVER) {
-        my $driver = driver(@_);
+    my $class = shift;
+    my $props = $class->properties || {};
+    unless ($props->{dbi_driver}) {
+        my $driver = $class->driver(@_);
         while ( $driver->can('fallback') ) {
             if ($driver->fallback) {
                 $driver = $driver->fallback;
@@ -871,9 +877,9 @@ sub dbi_driver {
                 last;
             }
         }
-        $DBI_DRIVER = $driver;
+        $props->{dbi_driver} = $driver;
     }
-    return $DBI_DRIVER;
+    return $props->{dbi_driver};
 }
 
 sub table_name {
@@ -887,6 +893,10 @@ sub clone_all {
     if ($clone->properties->{meta_installed}) {
         $clone->init_meta();
         $clone->meta( $obj->meta );
+        for my $meta ( keys %{ $clone->{__meta}->{__objects} } ) {
+            $clone->{__meta}->{__objects}{$meta}->{changed_cols}
+                = $obj->{__meta}->{__objects}->{$meta}->{changed_cols} || {};
+        }
     }
     return $clone;
 }
@@ -1083,21 +1093,25 @@ sub inflate {
     return $obj;
 }
 
-# We override D::OD's set_values method here only allowing the
-# assignment of a column if the value given is defined. There are
-# some legacy reasons for doing this, mostly for backward
-# compatibility.
 sub set_values {
     my $obj = shift;
     my ($values) = @_;
     for my $col (keys %$values) {
-        unless ( $obj->has_column($col) ) {
-            Carp::croak("You tried to set inexistent column $col to value $values->{$col} on " . ref($obj));
+        if ( $obj->has_column($col) ) {
+            # there's no point in croaking here; just set the
+            # values that are defined; ignore any others
+            $obj->$col($values->{$col}) if defined $values->{$col};
         }
-        $obj->$col($values->{$col}) if defined $values->{$col};
     }
 }
 
+sub get_values {
+    my $obj = shift;
+    # returns a copy of column_values, as accessing column_values directly
+    # can be problematic.
+    return { %{ $obj->column_values } };
+}
+
 sub column_def {
     my $obj = shift;
     my ($name) = @_;
@@ -1198,7 +1212,7 @@ sub clear_cache {
 
     if (@_) {
         $oc = $oc->{$key};
-        delete $oc->{shift} if $oc;
+        delete $oc->{ $_[0] } if $oc;
     } else {
         delete $oc->{$key};
     }
@@ -1210,7 +1224,7 @@ sub to_hash {
     my $hash = {};
     my $props = $obj->properties;
     my $pfx = $obj->datasource;
-    my $values = $obj->column_values;
+    my $values = $obj->get_values;
     foreach (keys %$values) {
         $hash->{"${pfx}.$_"} = $values->{$_};
     }
@@ -1248,6 +1262,7 @@ sub search_by_meta {
     };
     my $meta_class = $class->meta_pkg;
     my $meta_pk = $meta_class->primary_key_tuple;
+    $meta_pk = [ $meta_pk->[0] ]; # we only need the first column, that's the id
     my @metaobjs = $meta_class->search(
         $meta_terms, { %$args, fetchonly => $meta_pk }
     );
@@ -1271,9 +1286,18 @@ sub install_properties {
     my ($props) = @_;
     $props->{column_defs}->{$_} ||= 'string'
         for @{ $props->{columns} };
+
+    $props->{get_driver} ||= sub {
+        require MT::ObjectDriverFactory;
+        my $coderef = MT::ObjectDriverFactory->driver_for_class($class);
+        $class->get_driver($coderef);
+        return $coderef->(@_);
+    };
+
     $class->SUPER::install_properties(@_);
 }
 
+sub __properties { }
 sub meta_pkg { undef }
 
 *table_name = \&MT::Object::table_name;
@@ -1284,8 +1308,7 @@ sub meta_pkg { undef }
 *__parse_def = \&MT::Object::__parse_def;
 *count = \&MT::Object::count;
 *columns_of_type = \&MT::Object::columns_of_type;
-
-*driver = \&MT::Object::dbi_driver;
+*join_on = \&MT::Object::join_on;
 
 # TODO: copy this too
 sub blob_requires_zip {}
@@ -1362,6 +1385,8 @@ by declaring your class, and inheriting from I<MT::Object>:
 
     use base 'MT::Object';
 
+=over 4
+
 =item * __PACKAGE__->install_properties($args)
 
 Then call the I<install_properties> method on your class name; an easy way
@@ -1555,6 +1580,8 @@ classed objects (see 'class_type' above).
 
 =back
 
+=back
+
 =head1 USAGE
 
 =head2 System Initialization
@@ -2083,6 +2110,21 @@ names to their values in this object. For example:
 
 =back
 
+C<get_values> is similar to C<column_values>, in that it returns a hash
+reference of column names and values, but it returns a new hash reference,
+and a copy of the data rather that the hash reference that MT::Object uses
+internally. This is a safer way to access this data, if you intend to also
+modify the values.
+
+    my $data = $obj->get_values;
+    $data->{$_} = lc $data->{$_} for keys %$data;
+
+=over 4
+
+=item * $obj->set_values()
+
+=back
+
 C<set_values> accepts a similar hash ref, which need not give a value
 for every field. For example:
 
@@ -2223,7 +2265,7 @@ propery.
 This method returns all the C<column_defs> of the property of the
 object.
 
-=item Class->index_defs()
+=item * Class->index_defs()
 
 This method returns all the index definitions assigned to this class.
 This is the 'indexes' member of the properties installed for the class.
