@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: ObjectAsset.pm 1731 2008-04-02 23:27:20Z bchoate $
+# $Id: ObjectAsset.pm 3729 2009-05-15 16:04:57Z jmarcotte $
 
 package MT::ObjectAsset;
 
@@ -23,6 +23,10 @@ __PACKAGE__->install_properties({
         blog_obj => {
             columns => ['blog_id', 'object_ds', 'object_id'],
         },
+		# TODO: Figure out how we benefit from this (from Percona-Advance recommendation)
+		id_ds => {
+			columns => ['object_id', 'object_ds'],
+		},
         asset_id => 1,
     },
     defaults => {
