@@ -1,14 +1,18 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: DBI.pm 3067 2008-09-25 22:44:13Z bchoate $
+# $Id: DBI.pm 3783 2009-06-03 13:46:10Z fumiakiy $
 
 package MT::ObjectDriver::Driver::DBI;
 
 use strict;
 use base qw( Data::ObjectDriver::Driver::DBI );
 
+sub PING_CHECK_THROTTLE () { 5 }
+
+__PACKAGE__->mk_accessors(qw( role ));
+
 sub init {
     my $driver = shift;
     my (%param) = @_;
@@ -20,6 +24,49 @@ sub init {
     $driver;
 }
 
+sub start_query {
+    my $driver = shift;
+    my ($sql, $bind) = @_;
+    if ($MT::DebugMode && $MT::DebugMode & 4) {
+        $sql =~ s/\r?\n/ /g;
+        warn "QUERY: $sql";
+    }
+    return $driver->SUPER::start_query(@_);
+}
+
+sub dbh_handle {
+    my $driver = shift;
+    my ($opt) = @_;
+    my $dbh = $driver->dbh;
+    if ($dbh) {
+        if ( !$dbh->{private_last_ping} || ($dbh->{private_last_ping} + PING_CHECK_THROTTLE < time) ) {
+            if ( $dbh->ping ) {
+                $dbh->{private_last_ping} = time;
+            } else {
+                $driver->dbh($dbh = undef);
+            }
+        }
+    }
+    unless ($dbh) {
+        if (my $getter = $driver->get_dbh) {
+            local $opt->{driver} = $driver;
+            $dbh = $getter->($opt);
+            $dbh->{private_last_ping} = time if $dbh;
+        } else {
+            $dbh = $driver->init_db() or die $driver->last_error;
+            $dbh->{private_last_ping} = time;
+            $driver->dbh($dbh);
+        }
+    }
+    $dbh;
+}
+*rw_handle = \&dbh_handle;
+
+sub r_handle {
+    my $driver = shift;
+    return $driver->dbh_handle( { readonly => 1 } );
+}
+
 sub configure {
     my $driver = shift;
     $driver->dbd->configure($driver, @_);
@@ -86,6 +133,18 @@ sub remove_all {
     return $driver->direct_remove($class);
 }
 
+sub direct_remove {
+    my $driver = shift;
+    my ($class, $orig_terms, $orig_args) = @_;
+    $class->call_trigger('pre_direct_remove', $orig_terms, $orig_args);
+    $driver->SUPER::direct_remove(@_);
+}
+
+sub search {
+    my $driver = shift;
+    $driver->SUPER::search(@_);
+}
+
 sub count_group_by {
     my $driver = shift;
     my ($class, $terms, $args) = @_;
@@ -588,6 +647,8 @@ For example, in count_group_by method, $value holds the count for each
 group, while in sum_group_by method, $value holds the sum for each group.
 @returnvals parameter holds the additional data that wiil be retured.
 
+=back
+
 =head1 AUTHOR & COPYRIGHT
 
 Please see L<MT/AUTHOR & COPYRIGHT>.
