@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Pg.pm 1174 2008-01-08 21:02:50Z bchoate $
+# $Id: Pg.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::ObjectDriver::SQL::Pg;
 
@@ -26,4 +26,22 @@ sub as_limit {
            ($o ? " OFFSET " . int($o) : "");
 }
 
+sub _mk_term {
+    my $stmt = shift;
+    my ($col, $val) = @_;
+
+    if (ref $val eq 'HASH') {
+        if (!exists $val->{op}) {
+            if (exists $val->{like}) {
+                my $cols = $stmt->binary;
+                if (!$cols || !exists $cols->{$col}) {
+                    $val = { op => 'ILIKE', value => $val->{like} };
+                }
+            }
+        }
+    }
+
+    $stmt->SUPER::_mk_term($col, $val);
+}
+
 1;
