@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: mysql.pm 1502 2008-03-11 07:56:17Z fumiakiy $
+# $Id: mysql.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::ObjectDriver::SQL::mysql;
 
@@ -24,11 +24,28 @@ sub new {
             } else {
                 $sql->transform->{$col} = "binary($col)";
             }
+            $sql->transform->{$col . '__NOBINARY'} = $col;
         }
     }
     return $sql;
 }
 
+sub _mk_term {
+    my $stmt = shift;
+    my ($col, $val) = @_;
+
+    if ( my $transform = $stmt->transform ) {
+        my ($table_name, $col_name) = $col =~ m{ \A mt_(\w+)\.(\w+) }xms;
+        if ( $table_name ) {
+            my $key = join( '_', $table_name, $col_name );
+            if ( $transform->{$key . '__NOBINARY'} ) {
+                $stmt->add_where($col . '__NOBINARY', $val);
+            }
+        }
+    }
+    return $stmt->SUPER::_mk_term(@_);
+}
+
 sub add_freetext_where {
     my $stmt = shift;
     my ( $columns, $search_string ) = @_;
