@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Tag.pm 3074 2008-09-29 03:29:39Z fumiakiy $
+# $Id: Tag.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::Tag;
 
@@ -273,7 +273,7 @@ sub cache {
                     })
                 }),
                 limit => $limit,
-                fetchonly => [ 'name' ]
+                fetchonly => [ 'id', 'name' ]
             }
         );
         if (@tags) {
