@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Template.pm 3078 2008-09-30 03:21:03Z fumiakiy $
+# $Id: Template.pm 3474 2009-02-27 20:23:48Z bchoate $
 
 package MT::Template;
 
@@ -232,6 +232,8 @@ sub build {
     my $page_layout;
     if (my $blog_id = $tmpl->blog_id) {
         $ctx->stash('blog_id', $blog_id);
+        $ctx->stash('local_blog_id', $blog_id)
+            unless $ctx->stash('local_blog_id');
         my $blog = $ctx->stash('blog');
         unless ($blog) {
             $blog = MT::Blog->load($blog_id) or
@@ -240,6 +242,8 @@ sub build {
             $ctx->stash('blog', $blog);
         } else {
             $ctx->stash('blog_id', $blog->id);
+            $ctx->stash('local_blog_id', $blog->id)
+                unless $ctx->stash('local_blog_id');
         }
         MT->config->TimeOffset($blog->server_offset);
         $page_layout = $blog->page_layout;
@@ -317,7 +321,7 @@ sub save_widgetset {
         $obj->modulesets( join ',', @inst );
     }
     else {
-        @inst = split /,/, $obj->modulesets;
+        @inst = split /,/, ($obj->modulesets || '');
     }
 
     my @widgets = MT::Template->load(
@@ -445,7 +449,8 @@ sub text {
         }
         $tmpl->reset_tokens;
     }
-    $text;
+    $text = '' unless defined $text;
+    return $text;
 }
 
 sub _resync_to_db {
@@ -462,6 +467,19 @@ sub _resync_to_db {
 sub _sync_from_disk {
     my $tmpl = shift;
     my $lfile = $tmpl->linked_file;
+    if ($lfile eq '*') {
+        require MT::DefaultTemplates;
+        my $blog = $tmpl->blog;
+        my $set = MT::DefaultTemplates->templates( $blog ? $blog->template_set : () );
+        my ($set_tmpl) = grep { ($_->{type} eq $tmpl->type) && ($_->{identifier} eq $tmpl->identifier) } @$set;
+        if ($set_tmpl) {
+            my $c = $set_tmpl->{text};
+            $c = '' unless defined $c;
+            $c = MT->translate_templatized($c) if $c =~ m/<(?:mt|_)_trans\b/i;
+            return $c;
+        }
+        return;
+    }
     unless (File::Spec->file_name_is_absolute($lfile)) {
         if ($tmpl->blog_id) {
             my $blog = MT::Blog->load($tmpl->blog_id)
@@ -483,13 +501,14 @@ sub _sync_from_disk {
     close FH;
     $tmpl->linked_file_size($size);
     $tmpl->linked_file_mtime($mtime);
-    $c;
+    return $c;
 }
 
 sub _sync_to_disk {
     my $tmpl = shift;
     my($text) = @_;
     my $lfile = $tmpl->linked_file;
+    return 1 if $lfile eq '*';
     my $cfg = MT->config;
     if ($cfg->SafeMode) {
         ## Check for a set of extensions that aren't allowed.
