@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: Search.pm 2833 2008-07-23 18:52:51Z bchoate $
+# $Id: Search.pm 3455 2009-02-23 02:29:31Z auno $
 
 package MT::Template::Context::Search;
 
@@ -24,7 +24,9 @@ sub load_core_tags {
             SearchResults => \&_hdlr_results,
             'IfTagSearch?' => sub { MT->instance->mode eq 'tag' },
             'IfStraightSearch?' => sub { MT->instance->mode eq 'default' },
-            'NoSearchResults?' => sub { $_[0]->stash('count') ? 0 : 1; },
+            'NoSearchResults?' => sub { ( $_[0]->stash('search_string') &&
+                                   $_[0]->stash('search_string') =~ /\S/ &&
+                                   !$_[0]->stash('count') ) ? 1 : 0; },
             'NoSearch?' => sub { ( $_[0]->stash('search_string') &&
                                    $_[0]->stash('search_string') =~ /\S/ ) ? 0 : 1 },
             SearchResultsHeader => \&MT::Template::Context::_hdlr_pass_tokens,
@@ -339,7 +341,9 @@ sub _hdlr_results {
         #    $blog_footer = 1;
         #}
         if ( $next_object = $iter->() ) {
-            $blog_footer = $next_object->blog_id ne $this_object->blog_id ? 1 : 0;
+            if ( $next_object->can('blog') ) {
+                $blog_footer = $next_object->blog_id ne $this_object->blog_id ? 1 : 0;
+            }
         }
         else {
             $blog_footer = 1;
