@@ -1,8 +1,8 @@
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: WeblogPublisher.pm 3050 2008-09-12 06:57:45Z takayama $
+# $Id: WeblogPublisher.pm 3669 2009-05-06 18:06:12Z jmarcotte $
 
 package MT::WeblogPublisher;
 
@@ -504,8 +504,8 @@ sub rebuild_deleted_entry {
                     ArchiveType => $at,
                     Blog        => $blog->id,
                     (
-                        $archiver->author_based()
-                        ? ( author_id => $entry->author->id )
+                        $archiver->author_based() && $entry->author_id
+                        ? ( author_id => $entry->author_id )
                         : ()
                     ),
                     ( $archiver->date_based() ? ( startdate => $start ) : () ),
@@ -519,21 +519,21 @@ sub rebuild_deleted_entry {
             }
             else {
                 if ( $app->config('RebuildAtDelete') ) {
-                    if ( $archiver->author_based() ) {
+                    if ( $archiver->author_based() && $entry->author_id ) {
                         if ( $archiver->date_based() ) {
-                            $rebuild_recip{$at}{ $entry->author->id }
+                            $rebuild_recip{$at}{ $entry->author_id }
                               { $start . $end }{'Start'} = $start;
-                            $rebuild_recip{$at}{ $entry->author->id }
+                            $rebuild_recip{$at}{ $entry->author_id }
                               { $start . $end }{'End'} = $end;
-                            $rebuild_recip{$at}{ $entry->author->id }
+                            $rebuild_recip{$at}{ $entry->author_id }
                               { $start . $end }{'File'} =
                               MT::Util::archive_file_for( $entry, $blog, $at,
                                 undef, undef, undef, $entry->author );
                         }
                         else {
-                            $rebuild_recip{$at}{ $entry->author->id }{id} =
-                              $entry->author->id;
-                            $rebuild_recip{$at}{ $entry->author->id }{'File'} =
+                            $rebuild_recip{$at}{ $entry->author_id }{id} =
+                              $entry->author_id;
+                            $rebuild_recip{$at}{ $entry->author_id }{'File'} =
                               MT::Util::archive_file_for( $entry, $blog, $at,
                                 undef, undef, undef, $entry->author );
                         }
@@ -1099,6 +1099,7 @@ sub rebuild_file {
     local $ctx->{current_timestamp_end} = $end   if $end;
 
     $ctx->{__stash}{blog} = $blog;
+    $ctx->{__stash}{local_blog_id} = $blog->id;
 
     require MT::FileInfo;
 
@@ -1950,7 +1951,8 @@ sub archive_file_for {
             $cache->{ $blog->id . $at } = $map if $map;
         }
     }
-    my $file_tmpl = $map->file_template if $map;
+    my $file_tmpl;
+    $file_tmpl = $map->file_template if $map;
     unless ($file_tmpl) {
         if ( my $tmpls = $archiver->default_archive_templates ) {
             my ($default) = grep { $_->{default} } @$tmpls;
