@@ -1,9 +1,9 @@
 <?php
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: archive_lib.php 2304 2008-05-12 06:16:02Z takayama $
+# $Id: archive_lib.php 3455 2009-02-23 02:29:31Z auno $
 
 require_once("MTUtil.php");
 
@@ -260,8 +260,9 @@ class DateBasedArchiver extends BaseArchiver {
     function template_params(&$ctx) {
         parent::template_params($ctx);
         $vars =& $ctx->__stash['vars'];
-        $vars['archive_listing']  = 1;
-        $vars['archive_template'] = 1;
+        $vars['archive_listing']   = 1;
+        $vars['archive_template']  = 1;
+        $vars['datebased_archive'] = 1;
     }
 }
 
@@ -607,7 +608,7 @@ class AuthorBasedArchiver extends BaseArchiver {
         $vars['author_archive']                 = 1;
         $vars['archive_class']                  = 'author-archive';
         $vars['module_author-monthly_archives'] = 1;
-	$vars['archive_listing']                = 1;
+        $vars['archive_listing']                = 1;
     }
 }
 
@@ -965,7 +966,7 @@ class MonthlyAuthorBasedArchiver extends DateBasedAuthorArchiver {
         parent::template_params($ctx);
         $vars =& $ctx->__stash['vars'];
         $vars['archive_class']                  = 'author-monthly-archive';
-	$vars['author_monthly_archive']         = 1;
+        $vars['author_monthly_archive']         = 1;
         $vars['module_author-monthly_archives'] = 1;
     }
 }
@@ -1553,7 +1554,7 @@ class MonthlyCategoryArchiver extends DateBasedCategoryArchiver {
     function template_params(&$ctx) {
         parent::template_params($ctx);
         $vars =& $ctx->__stash['vars'];
-        $vars['archive_class']                    = 'category-monthlyl-archive';
+        $vars['archive_class']                    = 'category-monthly-archive';
         $vars['category_monthly_archive']         = 1;
         $vars['module_category-monthly_archives'] = 1;
     }
