@@ -1,5 +1,5 @@
 <?php
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -9,7 +9,7 @@ function smarty_block_mtifpingsmoderated($args, $content, &$ctx, &$repeat) {
     # status: complete
     if (!isset($content)) {
         $blog = $ctx->stash('blog');
-        $moderated = ($blog['blog_moderate_pings'];
+        $moderated = ($blog['blog_moderate_pings']);
         return $ctx->_hdlr_if($args, $content, $ctx, $repeat, $moderated);
     } else {
         return $ctx->_hdlr_if($args, $content, $ctx, $repeat);
