@@ -1,5 +1,5 @@
 <?php
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
@@ -15,18 +15,7 @@ function smarty_function_mtassetthumbnaillink($args, &$ctx) {
 
     require_once('MTUtil.php');
 
-    $width = 0;
-    $height = 0;
-    $scale = 0;
-
-    if (isset($args['width']))
-        $width = $args['width'];
-    if (isset($args['height']))
-        $height = $args['height'];
-    if (isset($args['scale']))
-        $scale = $args['scale'];
-
-    list($thumb, $thumb_w, $thumb_h) = get_thumbnail_file($asset, $blog, $width, $height, $scale);
+    list($thumb, $thumb_w, $thumb_h) = get_thumbnail_file($asset, $blog, $args);
     if (empty($thumb)) {
         return '';
     }
