@@ -1,9 +1,9 @@
 <?php
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: function.mtbloghost.php 1174 2008-01-08 21:02:50Z bchoate $
+# $Id: function.mtbloghost.php 3455 2009-02-23 02:29:31Z auno $
 
 function smarty_function_mtbloghost($args, &$ctx) {
     // status: complete
@@ -13,7 +13,7 @@ function smarty_function_mtbloghost($args, &$ctx) {
     if (!preg_match('!/$!', $host))
         $host .= '/';
 
-    if (preg_match('!^https?://([^/:]+)(:\d+)?/!', $host, $matches)) {
+    if (preg_match('!^https?://([^/:]+)(:\d+)?/?!', $host, $matches)) {
         if ($args['signature']) {
             $sig = $matches[1];
             $sig = preg_replace('/\./', '_', $sig);
@@ -24,4 +24,3 @@ function smarty_function_mtbloghost($args, &$ctx) {
         return '';
     }
 }
-?>
