@@ -1,14 +1,13 @@
 <?php
-# Movable Type (r) Open Source (C) 2001-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2001-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: function.mtsubcategorypath.php 1174 2008-01-08 21:02:50Z bchoate $
+# $Id: function.mtsubcategorypath.php 3455 2009-02-23 02:29:31Z auno $
 
 function smarty_function_mtsubcategorypath($args, &$ctx) {
     require_once("block.mtparentcategories.php");
-    require_once("function.mtcategorylabel.php");
-    require_once("modifier.dirify.php");
+    require_once("function.mtcategorybasename.php");
 
     $args = array('glue' => '/');
     $content = null;
@@ -16,8 +15,7 @@ function smarty_function_mtsubcategorypath($args, &$ctx) {
     smarty_block_mtparentcategories($args, $content, $ctx, $repeat);
     $res = '';
     while ($repeat) {
-        $content = smarty_function_mtcategorylabel(array(), $ctx);
-        $content = smarty_modifier_dirify($content, isset($args['separator']) ? $args['separator'] : '1');
+        $content = smarty_function_mtcategorybasename(array(), $ctx);
         $res .= smarty_block_mtparentcategories($args, $content, $ctx, $repeat);
     }
     return $res;
