@@ -1,13 +1,13 @@
 <?php
-# Movable Type (r) Open Source (C) 2004-2008 Six Apart, Ltd.
+# Movable Type (r) Open Source (C) 2004-2009 Six Apart, Ltd.
 # This program is distributed under the terms of the
 # GNU General Public License, version 2.
 #
-# $Id: mt.php.pre 2688 2008-07-03 00:38:58Z bchoate $
+# $Id: mt.php.pre 3536 2009-03-12 12:57:34Z fumiakiy $
 
-define('VERSION', '4.23');
-define('VERSION_ID', '4.23-en');
-define('PRODUCT_VERSION', '4.23');
+define('VERSION', '4.261');
+define('VERSION_ID', '4.261');
+define('PRODUCT_VERSION', '4.261');
 define('PRODUCT_NAME', 'Movable Type');
 
 global $Lexicon;
@@ -196,6 +196,7 @@ class MT {
 
         $cfg = array();
         $type_array = array('pluginpath', 'alttemplate', 'outboundtrackbackdomains', 'memcachedservers');
+        $type_hash  = array('commenterregistration');
         if ($fp = file($file)) {
             foreach ($fp as $line) {
                 // search through the file
@@ -206,6 +207,10 @@ class MT {
                         $value = trim($regs[2]);
                         if (in_array($key, $type_array)) {
                             $cfg[$key][] = $value;
+                        }
+                        elseif (in_array($key, $type_hash)) {
+                            $hash = preg_split('/\=/', $value, 2);
+                            $cfg[$key][strtolower(trim($hash[0]))] = trim($hash[1]);
                         } else {
                             $cfg[$key] = $value;
                         }
@@ -291,7 +296,7 @@ class MT {
                     if (preg_match('/^\s*(\S+)\s+(.*)$/', $line, $regs)) {
                         $key = strtolower(trim($regs[1]));
                         $value = trim($regs[2]);
-                        if (($key == 'PluginSwitch') || ($key == 'PluginSchemaVersion')) { # special case for hash
+                        if (($key === 'pluginswitch') || ($key === 'pluginschemaversion')) { # special case for hash
                             if (preg_match('/^(.+)=(.+)$/', $value, $match))
                                 $cfg[$key][trim($match[1])] = trim($match[2]);
                         } else {
@@ -381,6 +386,8 @@ class MT {
             $cfg['usersessioncookiepath'] = 'DEFAULT';
         isset($cfg['usersessioncookietimeout']) or
             $cfg['usersessioncookietimeout'] = 60*60*4;
+        isset($cfg['commenterregistration']) or
+            $cfg['commenterregistration'] = array('allow' => 1 );
     }
 
     function configure_paths($blog_site_path) {
@@ -507,6 +514,7 @@ class MT {
 
         // start populating our stash
         $ctx->stash('blog_id', $blog_id);
+        $ctx->stash('local_blog_id', $blog_id);
         $ctx->stash('blog', $blog);
         $ctx->stash('build_template_id', $tpl_id);
 
@@ -567,11 +575,11 @@ class MT {
 
             if ($at == 'Category') {
                 $vars =& $ctx->__stash['vars'];
-                $vars['archive_class']                    = "category-archive";
-                $vars['category_archive']                 = 1;
-                $vars['archive_template']                 = 1;
-                $vars['archive_listing']                  = 1;
-                $vars['module_category-monthly_archives'] = 1;
+                $vars['archive_class']            = "category-archive";
+                $vars['category_archive']         = 1;
+                $vars['archive_template']         = 1;
+                $vars['archive_listing']          = 1;
+                $vars['module_category_archives'] = 1;
             }
         }
 
@@ -675,6 +683,7 @@ class MT {
             $blog =& $db->fetch_blog($this->blog_id);
             $ctx->stash('blog', $blog);
             $ctx->stash('blog_id', $this->blog_id);
+            $ctx->stash('local_blog_id', $this->blog_id);
             $this->configure_paths($blog['blog_site_path']);
         }
         return $ctx->display($tpl, $cid);
@@ -690,6 +699,7 @@ class MT {
             $blog =& $db->fetch_blog($this->blog_id);
             $ctx->stash('blog', $blog);
             $ctx->stash('blog_id', $this->blog_id);
+            $ctx->stash('local_blog_id', $this->blog_id);
             $this->configure_paths($blog['blog_site_path']);
         }
         return $ctx->fetch($tpl, $cid);
@@ -727,6 +737,7 @@ class MT {
             $mtphpdir = $this->config('PHPDir');
             $ctx =& $this->context();
             $ctx->stash('blog_id', $this->blog_id);
+            $ctx->stash('local_blog_id', $this->blog_id);
             $ctx->stash('blog', $this->db->fetch_blog($this->blog_id));
             $ctx->stash('error_message', $errstr."<!-- file: $errfile; line: $errline; code: $errno -->");
             $ctx->stash('error_code', $errno);
